@@ -260,7 +260,8 @@ static void doAll(status, first, last, group)
 			     ret, "enterNewsgroup", "doAll");
 		}
 		else if (status == SUBSCRIBE) {
-		    subscribe();
+		  if (! subscribe())
+		    ret = BAD_GROUP;
 		}
 		else {
 		    unsubscribe();
@@ -347,7 +348,7 @@ static void allHandler(widget, client_data, call_data)
     inCommand = 1;
     xrnBusyCursor();
 
-    if ((int) client_data != XRN_CB_ABORT)
+    if ((POINTER_NUM_TYPE) client_data != XRN_CB_ABORT)
 	doAll(SUBSCRIBE, False, False, GetDialogValue(AllBox));
 
     PopDownDialog(AllBox);
@@ -486,11 +487,12 @@ static void limitHandler(widget, client_data, call_data)
     if (strcmp(DOIT_STRING, (char *) client_data) == 0) {
       FREE(LimitString);
 
-      if ((LimitString = GetDialogValue(LimitBox)))
+      if ((LimitString = GetDialogValue(LimitBox))) {
 	if (! *LimitString)
 	  LimitString = NULL;
 	else
 	  LimitString = XtNewString(LimitString);
+      }
 
       redrawAllWidget();
     }
@@ -573,6 +575,7 @@ void allGotoFunction(widget, event, string, count)
 	mesgPane(XRN_SERIOUS, 0, NO_SUCH_NG_DELETED_MSG, newGroup);
     }
     else if (ret == XRN_NOMORE) {
+	exitNewsgroup();
 	mesgPane(XRN_SERIOUS, 0, NO_ARTICLES_MSG, newGroup);
     }
     else {
