@@ -478,6 +478,10 @@ static int getNearbyArticle(status, file, question, artNum)
 	}
 	move = True;
 
+	/* If this assertion fires, you probably need to put "#define
+	   XAW_REDISPLAY_BUG" in config.h and recompile XRN. */
+	assert(ArtPosition == 0 || SubjectString[ArtPosition - 1] == '\n');
+
 	if (! SubjectString[ArtPosition])
 	    continue;
 
@@ -486,6 +490,10 @@ static int getNearbyArticle(status, file, question, artNum)
 
 	*artNum = atol(&SubjectString[ArtPosition + 2]);
 
+	/* If this assertion fires, you probably need to put "#define
+	   XAW_REDISPLAY_BUG" in config.h and recompile XRN. */
+	assert(*artNum);
+
 	if (getArticle(CurrentGroup, *artNum, file, question) != XRN_OKAY) {
 	    mesgPane(XRN_SERIOUS, mesg_name, ART_NOT_AVAIL_MSG, *artNum);
 	    TextRemoveLine(SubjectText, ArtPosition);
@@ -633,6 +641,7 @@ static int isPrevSubject(subject, file, question, artNum)
 
 	    art = artStructGet(CurrentGroup, *artNum, True);
 	    SET_MAYBE_LISTED(art);
+	    artStructSet(CurrentGroup, &art);
 
 	    if (newSubjects) {
 		tmp = XtMalloc(utStrlen(newSubjects) + utStrlen(newLine) + 1);
@@ -858,6 +867,8 @@ int switchToArticleMode()
 	SET_LISTED(&copy);
 	artStructReplace(newsgroup, &art, &copy, i);
       }
+      else
+	ART_STRUCT_UNLOCK;
     }
 
     SubjectString = getSubjects(TextGetColumns(SubjectText), FirstListedArticle,
@@ -1008,6 +1019,8 @@ static void foundArticle(file, ques, motion)
 
 	setButtonSensitive(SubjectButtonBox, "artLast", True);
 
+	ART_STRUCT_UNLOCK;
+
 	if (art->file)
 	  file_cache_file_unlock(FileCache, *art->file);
     }
@@ -1498,40 +1511,27 @@ void artNextGroupFunction(widget, event, string, count)
 	    mesgPane(XRN_SERIOUS, mesg_name, NO_SUCH_NG_DELETED_MSG, name);
 	    mesgPane(XRN_SERIOUS | XRN_SAME_LINE, mesg_name,
 		     SKIPPING_TO_NEXT_NG_MSG);
+	save_name:
 	    CurrentIndexGroup = XtRealloc(CurrentIndexGroup, strlen(name) + 1);
 	    (void) strcpy(CurrentIndexGroup, name);
 	    continue;
 	}
-	else if (ret == XRN_NOMORE) {
+	else if ((ret == XRN_NOMORE) || (ret == XRN_NOUNREAD)) {
+	    exitNewsgroup();
 	    if ((p = strstr(&ArticleNewsGroupsString[GroupPosition],
-			    NEWS_IN_MSG)) &&
+			    (ret == XRN_NOMORE) ? NEWS_IN_MSG : UNREAD_MSG)) &&
 		(p < strstr(&ArticleNewsGroupsString[GroupPosition], name))) {
-		mesgPane(XRN_INFO, mesg_name, PROBABLY_EXPIRED_MSG, name);
+		mesgPane(XRN_INFO, mesg_name, (ret == XRN_NOMORE) ?
+			 PROBABLY_EXPIRED_MSG : PROBABLY_KILLED_MSG, name);
 		mesgPane(XRN_INFO | XRN_SAME_LINE, mesg_name,
 			 SKIPPING_TO_NEXT_NG_MSG);
 	    }
-	    CurrentIndexGroup = XtRealloc(CurrentIndexGroup, strlen(name) + 1);
-	    (void) strcpy(CurrentIndexGroup, name);
-	    continue;
-	}
-	else if (ret == XRN_NOUNREAD) {
-	    if ((p = strstr(&ArticleNewsGroupsString[GroupPosition],
-			    UNREAD_MSG)) &&
-		(p < strstr(&ArticleNewsGroupsString[GroupPosition], name))) {
-		mesgPane(XRN_INFO, mesg_name, PROBABLY_KILLED_MSG, name);
-		mesgPane(XRN_INFO | XRN_SAME_LINE, mesg_name,
-			 SKIPPING_TO_NEXT_NG_MSG);
-	    }
-	    CurrentIndexGroup = XtRealloc(CurrentIndexGroup, strlen(name) + 1);
-	    (void) strcpy(CurrentIndexGroup, name);
-	    continue;
+	    goto save_name;
 	}
 	else if (ret != GOOD_GROUP) {
 	    mesgPane(XRN_SERIOUS, mesg_name, UNKNOWN_FUNC_RESPONSE_MSG,
 		     ret, "enterNewsgroup", "artNextGroupFunction");
-	    CurrentIndexGroup = XtRealloc(CurrentIndexGroup, strlen(name) + 1);
-	    (void) strcpy(CurrentIndexGroup, name);
-	    continue;
+	    goto save_name;
 	}
 
 	 if (switchToArticleMode() == GOOD_GROUP) {
@@ -1955,9 +1955,14 @@ void artThreadParentFunction(widget, event, string, count)
   art = artStructGet(CurrentGroup, artNum, False);
   assert(art);
 
-  if (prefer_listed && (parent = art->parent) &&
-      articleIsAvailable(CurrentGroup, parent))
-    goto done;
+  if (prefer_listed && (parent = art->parent)) {
+    ART_STRUCT_UNLOCK;
+    if (articleIsAvailable(CurrentGroup, parent))
+      goto done;
+    art = artStructGet(CurrentGroup, artNum, False);
+  }
+
+  ART_STRUCT_UNLOCK;
 
   if (! ((refs = XtNewString(art->references)) && *refs))
     goto done;
@@ -1974,13 +1979,14 @@ void artThreadParentFunction(widget, event, string, count)
       /* empty */;
     if (begin_id < refs)
       break;
-    if ((parent = getArticleNumberFromIdXref(CurrentGroup, begin_id)))
+    if ((parent = getArticleNumberFromIdXref(CurrentGroup, begin_id))) {
       if ((parent < 0) || !articleIsAvailable(CurrentGroup, parent)) {
 	parent = 0;
 	continue;
       }
       else
 	break;
+    }
     if (! cancel_created) {
       abortClear();
       cancelCreate("CancelThreadParent");
@@ -1992,10 +1998,11 @@ void artThreadParentFunction(widget, event, string, count)
       (void) getidlist(CurrentGroup, block_start, block_end, False, 0);
       for (checkee = block_end ; checkee >= block_start; checkee--) {
 	art = artStructGet(CurrentGroup, checkee, False);
+	ART_STRUCT_UNLOCK;
 	if (art->id && !strcmp(art->id, begin_id) &&
 	    articleIsAvailable(CurrentGroup, checkee)) {
 	  parent = checkee;
-	  break;
+	  goto done;
 	}
       }
       if (abortP()) {
@@ -2090,6 +2097,8 @@ static String artKillSession(
     assert(art);
 
     match_val = *(char **)((char *)art + source_offset);
+    ART_STRUCT_UNLOCK;
+
     if (source_fix_function)
       match_val = (*source_fix_function)(match_val);
     if (! match_val) {
@@ -2122,7 +2131,8 @@ static String artKillSession(
 
 	left = 0;
 	while (SubjectString[left] != '\0') {
-	    if (SubjectString[left] != UNREAD_MARKER) {
+	    if ((SubjectString[left] != UNREAD_MARKER) &&
+		(SubjectString[left] != READ_MARKER)) {
 		artNum = atol(&SubjectString[left+2]);
 		art = artStructGet(CurrentGroup, artNum, False);
 		assert(art);
@@ -2137,9 +2147,12 @@ static String artKillSession(
 		    SET_UNLISTED(&copy);
 		    artStructReplace(CurrentGroup, &art, &copy, artNum);
 		}
-		else if
+		else {
+		  ART_STRUCT_UNLOCK;
+
+		  if
 #ifdef POSIX_REGEX
-			(! regexec(&reStruct, cur_val, 0, 0, 0))
+		        (! regexec(&reStruct, cur_val, 0, 0, 0))
 #else
 # ifdef SYSV_REGEX
 	                (regex(reRet, cur_val))
@@ -2152,6 +2165,7 @@ static String artKillSession(
 			(void) markStringRead(SubjectString, left);
 			TextInvalidate(SubjectText, SubjectString, left, left + 1);
 		    }
+		}
 	    }
 	    if (!moveCursor(FORWARD, SubjectString, &left)) {
 		break;
@@ -2320,17 +2334,17 @@ void artListOldHandler(widget, client_data, call_data)
     art_num artNum;
     char *numberstr;
 
-    if (inCommand && ((int) client_data != XRNgotoArticle_DEFAULT))
+    if (inCommand && ((POINTER_NUM_TYPE) client_data != XRNgotoArticle_DEFAULT))
       return;
     inCommand = 1;
     xrnBusyCursor();
 
     TextUnsetSelection(SubjectText);
 
-    if ((int) client_data == XRNgotoArticle_ABORT) {
+    if ((POINTER_NUM_TYPE) client_data == XRNgotoArticle_ABORT) {
       goto finishedl;
     }
-    else if ((int) client_data == XRNgotoArticle_DEFAULT) {
+    else if ((POINTER_NUM_TYPE) client_data == XRNgotoArticle_DEFAULT) {
       artNum = CurrentGroup->first;
     }
     else {
@@ -2351,6 +2365,10 @@ void artListOldHandler(widget, client_data, call_data)
       }
     }
 
+    if (GotoArticleBox) {
+      PopDownDialog(GotoArticleBox);
+      GotoArticleBox = 0;
+    }
     abortClear();
     cancelCreate("CancelListOld");
 
@@ -2371,6 +2389,8 @@ void artListOldHandler(widget, client_data, call_data)
 	  artStructReplace(CurrentGroup, &art, &copy, i);
 	  FirstListedArticle = MIN(FirstListedArticle, i);
 	}
+	else
+	  ART_STRUCT_UNLOCK;
       }
       SubjectString = getSubjects(TextGetColumns(SubjectText),
 				  FirstListedArticle, True);
@@ -2394,10 +2414,6 @@ void artListOldHandler(widget, client_data, call_data)
     }
 
 finishedl:
-    if (GotoArticleBox) {
-      PopDownDialog(GotoArticleBox);
-      GotoArticleBox = 0;
-    }
     xrnUnbusyCursor();
     cancelDestroy();
     inCommand = 0;
@@ -2534,37 +2550,32 @@ static void gotoArticleHandler(widget, client_data, call_data)
     inCommand = 1;
     xrnBusyCursor();
     TextUnsetSelection(SubjectText);
-    if ((int) client_data == XRNgotoArticle_ABORT) {
-	PopDownDialog(GotoArticleBox);
-	GotoArticleBox = 0;
-	xrnUnbusyCursor();
-	inCommand = 0;
-	return;
-    }
     numberstr = GetDialogValue(GotoArticleBox);
+    PopDownDialog(GotoArticleBox);
+    GotoArticleBox = 0;
+    if ((POINTER_NUM_TYPE) client_data == XRNgotoArticle_ABORT) {
+      goto finished;
+    }
+    abortClear();
+    cancelCreate("CancelGotoArticle");
     if (! (numberstr && *numberstr)) {
 	mesgPane(XRN_INFO, 0, NO_ART_NUM_MSG);
-	PopDownDialog(GotoArticleBox);
-	GotoArticleBox = 0;
-	xrnUnbusyCursor();
-	inCommand = 0;
-	return;
+	goto finished;
     }
 
     artNum = atol(numberstr);
     if (artNum == 0) {
 	mesgPane(XRN_SERIOUS, 0, BAD_ART_NUM_MSG, numberstr);
-	PopDownDialog(GotoArticleBox);
-	GotoArticleBox = 0;
-	xrnUnbusyCursor();
-	inCommand = 0;
-	return;
+	goto finished;
     }
 
     status = moveToArticle(CurrentGroup, artNum, &file, &question);
 
     switch (status) {
 
+    case ABORT:
+      break;
+
     case NOMATCH:
     case ERROR:
 	mesgPane(XRN_SERIOUS, 0, ART_NOT_AVAIL_MSG, artNum);
@@ -2598,10 +2609,10 @@ static void gotoArticleHandler(widget, client_data, call_data)
 	break;
     }
 
-    PopDownDialog(GotoArticleBox);
-    GotoArticleBox = 0;
+ finished:
     xrnUnbusyCursor();
     inCommand = 0;
+    cancelDestroy();
     return;
 }
 
@@ -2866,6 +2877,7 @@ static int subjectSearch(dir, position, expr, file, ques, artNum)
 
 		art = artStructGet(CurrentGroup, *artNum, True);
 		SET_MAYBE_LISTED(art);
+		artStructSet(CurrentGroup, &art);
 		if (! last_new)
 		  first_new = last_new = *artNum;
 		else
@@ -2884,6 +2896,7 @@ static int subjectSearch(dir, position, expr, file, ques, artNum)
 		    if (getArticle(CurrentGroup, *artNum, file, ques) != XRN_OKAY) {
 			mesgPane(XRN_SERIOUS, 0, ART_NOT_AVAIL_MSG, *artNum);
 			removeLine(newString, 0);
+			art = artStructGet(CurrentGroup, *artNum, True);
 			SET_UNLISTED(art);
 			artStructSet(CurrentGroup, &art);
 			continue;
@@ -2999,7 +3012,7 @@ static void subSearchHandler(widget, client_data, call_data)
     inCommand = 1;
     xrnBusyCursor();
 
-    if ((int) client_data == XRNsubSearch_ABORT) {
+    if ((POINTER_NUM_TYPE) client_data == XRNsubSearch_ABORT) {
 	do_search = False;
     }
     else {
@@ -3024,7 +3037,7 @@ static void subSearchHandler(widget, client_data, call_data)
     SubSearchBox = 0;
 
     if (do_search) {
-	direction = ((int) client_data == XRNsubSearch_FORWARD) ?
+	direction = ((POINTER_NUM_TYPE) client_data == XRNsubSearch_FORWARD) ?
 	    FORWARD : BACK;
 	LastSearch = direction;
 	doSubSearch(LastRegexp, direction);
@@ -3207,7 +3220,7 @@ static void saveHandler(widget, client_data, call_data)
     inCommand = 1;
     xrnBusyCursor();
 
-    if ((int) client_data != XRNsave_ABORT) {
+    if ((POINTER_NUM_TYPE) client_data != XRNsave_ABORT) {
 	template = GetDialogValue(SaveBox);
 	doSave(template, False);
 	if (SaveString && (SaveString != app_resources.saveString)) {
@@ -3573,8 +3586,8 @@ void displayArticleWidgets()
 	XawPanedAllowResize(TEXT_PANE_CHILD(SubjectText), True);
 	TextSetLines(SubjectText, app_resources.topLines);
 	XawPanedAllowResize(TEXT_PANE_CHILD(SubjectText), False);
-	XtVaGetValues(SubjectText, XtNheight, &height, 0);
-	XtVaSetValues(SubjectText, XtNpreferredPaneSize, height, 0);
+	XtVaGetValues(SubjectText, XtNheight, &height, (String)0);
+	XtVaSetValues(SubjectText, XtNpreferredPaneSize, height, (String)0);
 
 	TopInfoLine = SubjectInfoLine;
 	BottomInfoLine = ArticleInfoLine;
@@ -3626,8 +3639,10 @@ void artDoTheRightThing(widget, event, string, count)
 	    }
 	}
 	else {
+	    long top_position = TextGetTopPosition(ArticleText);
 	    artScrollFunction(widget, event, string, count);
-	    if (TextPastLastPage(ArticleText))
+	    if (TextPastLastPage(ArticleText) ||
+		(top_position == TextGetTopPosition(ArticleText)))
 		goto next_article;
 	}
     }
