@@ -7,6 +7,8 @@
 #include "artstruct.h"
 #include "file_cache.h"
 
+int art_struct_locked = 0;
+
 static Boolean artsame _ARGUMENTS((struct article *, struct article *));
 static void freeartstruct _ARGUMENTS((struct article *));
 
@@ -52,6 +54,52 @@ void artListSet(newsgroup)
     newsgroup->ref_art = newsgroup->articles;
 }
 
+/*
+  Extend a group's article structure, if it exists, from the old last
+  article number to the new one.
+*/
+void artListExtend(newsgroup, old_last)
+     struct newsgroup *newsgroup;
+     art_num old_last;
+{
+  struct article copy, *new, *ptr;
+
+  assert(! art_struct_locked);
+
+  if (! newsgroup->articles)
+    return;
+
+  if (old_last >= newsgroup->last)
+    return;
+
+  CLEAR_ALL_NO_FREE(&copy);
+
+  for (ptr = newsgroup->articles; ptr->next; ptr = ptr->next)
+    /* empty */;
+
+  if (artsame(&copy, ptr)) {
+#ifdef DEBUG
+    fprintf(stderr,
+	    "artListExtend: extending blank article structure from %d to %d\n",
+	    old_last, newsgroup->last);
+#endif
+    return;
+  }
+
+#ifdef DEBUG
+  fprintf(stderr, "artListExtend: creating new article structure for %d to %d\n",
+	  old_last, newsgroup->last);
+#endif
+
+  new = (struct article *) XtCalloc(1, sizeof(struct article));
+  CLEAR_ALL_NO_FREE(new);
+  new->first = old_last + 1;
+  new->previous = ptr;
+  new->next = 0;
+
+  ptr->next = new;
+}
+
 
 int artStructNumChildren(art)
      struct article *art;
@@ -172,6 +220,11 @@ struct article *artStructGet(
 {
     struct article *reference = newsgroup->ref_art;
 
+    assert(! art_struct_locked);
+    ART_STRUCT_LOCK;
+
+    assert(artnum <= newsgroup->last);
+
     if (! reference)
 	reference = newsgroup->articles;
 
@@ -217,6 +270,9 @@ struct article *artStructGet(
 	COPY_FIELD(references);
 	new->file = reference->file;
 	new->base_file = reference->base_file;
+	/* Not copying the headers; they will be regenerated if
+	   they're needed. */
+	new->headers = 0;
 	new->parent = reference->parent;
 	new->children = copy_art_list(reference);
 	new->first = artnum + 1;
@@ -246,6 +302,7 @@ struct article *artStructGet(
 	COPY_FIELD(references);
 	new->file = reference->file;
 	new->base_file = reference->base_file;
+	new->headers = 0;
 	new->parent = reference->parent;
 	new->children = copy_art_list(reference);
 	new->first = artnum;
@@ -284,6 +341,8 @@ void artStructSet(newsgroup, art)
     struct newsgroup *newsgroup;
     struct article **art;
 {
+    ART_STRUCT_UNLOCK;
+
     if (artsame(*art, (*art)->previous)) {
       struct article *tmp = *art;
 
@@ -354,6 +413,8 @@ void artStructReplace(newsgroup, original, copy, artnum)
     struct article *copy;
     art_num artnum;
 {
+    ART_STRUCT_UNLOCK;
+
     if (! artsame(*original, copy)) {
 	*original = artStructGet(newsgroup, artnum, True);
 	(*original)->status = copy->status;
@@ -373,6 +434,11 @@ void artStructReplace(newsgroup, original, copy, artnum)
 	    CLEAR_BASE_FILE(*original);
 	    CHECK(base_file, copy, *original);
 	}
+	if ((*original)->headers != copy->headers) {
+	  CLEAR_HEADERS(*original);
+	  (*original)->headers = copy->headers;
+	}
+	CHECK(headers, copy, *original);
 	CHECK(id, copy, *original);
 	CHECK(xref, copy, *original);
 	CHECK(approved, copy, *original);
@@ -395,23 +461,27 @@ struct article *artStructNext(newsgroup, art, first, last)
     struct article *art;
     art_num *first, *last;
 {
+    assert(art_struct_locked);
+
     art = art->next;
     if (! art) {
 	newsgroup->ref_art = 0;
 	return 0;
     }
 
-    if (first)
+    if (first) {
 	if (art->first)
 	    *first = art->first;
 	else
-	    *first = newsgroup->first;
+	  *first = newsgroup->first;
+    }
 
-    if (last)
+    if (last) {
 	if (art->next)
 	    *last = art->next->first - 1;
 	else
-	    *last = newsgroup->last;
+	  *last = newsgroup->last;
+    }
 
     newsgroup->ref_art = art;
     return art;
@@ -427,23 +497,27 @@ struct article *artStructPrevious(newsgroup, art, first, last)
     struct article *art;
     art_num *first, *last;
 {
+    assert(art_struct_locked);
+
     art = art->previous;
     if (! art) {
 	newsgroup->ref_art = 0;
 	return 0;
     }
 
-    if (first)
+    if (first) {
 	if (art->first)
 	    *first = art->first;
 	else
-	    *first = newsgroup->first;
+	  *first = newsgroup->first;
+    }
 
-    if (last)
+    if (last) {
 	if (art->next)
 	    *last = art->next->first - 1;
 	else
-	    *last = newsgroup->last;
+	  *last = newsgroup->last;
+    }
 
     newsgroup->ref_art = art;
     return art;
@@ -461,19 +535,24 @@ struct article *artListFirst(newsgroup, first, last)
 {
     struct article *art = newsgroup->articles;
 
+    assert(! art_struct_locked);
+
     if (! art) {
 	newsgroup->ref_art = 0;
 	return 0;
     }
 
+    ART_STRUCT_LOCK;
+
     if (first)
 	*first = art->first;
 
-    if (last)
+    if (last) {
 	if (art->next)
 	    *last = art->next->first - 1;
 	else
-	    *last = newsgroup->last;
+	  *last = newsgroup->last;
+    }
 
     newsgroup->ref_art = art;
     return art;
@@ -490,19 +569,24 @@ struct article *artListLast(newsgroup, first, last)
 {
     struct article *art = newsgroup->articles;
 
+    assert(! art_struct_locked);
+
     if (! art) {
 	newsgroup->ref_art = 0;
 	return 0;
     }
 
+    ART_STRUCT_LOCK;
+
     while (art->next)
 	art = art->next;
 
-    if (first)
+    if (first) {
 	if (art->first)
 	    *first = art->first;
 	else
-	    *first = newsgroup->first;
+	  *first = newsgroup->first;
+    }
 
     if (last)
 	*last = newsgroup->last;
@@ -556,6 +640,9 @@ static Boolean artsame(art1, art2)
     if (art1->base_file != art2->base_file)
       return False;
 
+    if (art1->headers != art2->headers)
+      return False;
+
     if (art1->parent != art2->parent)
       return False;
 
