@@ -1,7 +1,3 @@
-#if !defined(lint) && !defined(SABER) && !defined(GCC_WALL)
-static char XRNrcsid[] = "$Id: compose.c,v 1.126 1999/09/24 14:00:55 jik Exp $";
-#endif
-
 /*
  * xrn - an X-based NNTP news reader
  *
@@ -31,6 +27,8 @@ static char XRNrcsid[] = "$Id: compose.c,v 1.126 1999/09/24 14:00:55 jik Exp $";
  * compose.c: routines for composing messages and articles
  */
 
+#include <stdio.h>
+
 #include "copyright.h"
 #include "config.h"
 #include "utils.h"
@@ -88,18 +86,11 @@ extern int fclose _ARGUMENTS((FILE *));
 #include "server.h"
 #include "mesg.h"
 #include "xrn.h"
-#include "patchlevel.h"
 #include "xmisc.h"
 #include "buttons.h"
 #include "butdefs.h"
 #include "mesg_strings.h"
 
-#ifdef INN
-#include <configdata.h>
-#include <clibrary.h>
-#include <libinn.h>
-#endif
-
 struct header {
     struct newsgroup *newsgroup;
     art_num article;
@@ -123,9 +114,7 @@ struct header {
 #ifndef INEWS
     char *sender_host;
 #endif
-#ifndef INN
     char *path;
-#endif
     char *organization;
 
     char  *date; /* added to header structure....... */
@@ -140,7 +129,7 @@ static char *followup_or_reply_title _ARGUMENTS((struct header *, int, int));
 static void compose_buttons _ARGUMENTS((Widget, int));
 static int check_quoted_text _ARGUMENTS((char *));
 static char *insert_courtesy_tag _ARGUMENTS((char *));
-static Boolean IsValidAddressField _ARGUMENTS((CONST char *));
+static Boolean IsValidAddressField _ARGUMENTS((const char *));
 
 #ifdef GENERATE_EXTRA_FIELDS
 static char *gen_id _ARGUMENTS((void));
@@ -272,14 +261,13 @@ char *getUser()
   host:
   * Contents of the HIDDENHOST environment variable, if it's set, or
   * Contents of the hiddenHost X resource, if it's set, or
-  * INN "fromhost" configuration value, if using INN and it's set, or
   * Contents of the file HIDDEN_FILE, if it's defined and non-empty, or
   * The string RETURN_HOST, if it's defined, or
   * Contents of real_host
   If the derived value for host doesn't have a period in it, the
   domain is determined (see below) and appended to it.
 
-  path: (not used under INN)
+  path:
   * Contents of the HIDDENPATH environment variable, if it's set, or
   * Contents of the file PATH_FILE, if it's defined and non-empty, or
   * Contents of real_host
@@ -287,7 +275,6 @@ char *getUser()
   host's domain: (only if host doesn't already have a period in it)
   * Contents of the "domainName" X resource, if it's set, or
   * Contents of the DOMAIN environment variable, if it's set, or
-  * INN "domain" configuration value, if using INN and it's set, or
   * Contents of the file DOMAIN_FILE, if it's defined and non-empty, or
   * The string DOMAIN_NAME, if it's defined, or
   * Error signalled and post/mail aborted
@@ -332,18 +319,22 @@ static int getHeader(article, Header)
 #ifndef INEWS
   char sender_host[HOST_NAME_SIZE];
 #endif
-#ifdef INN
-  char *inn_org;
-#else
   char path[HOST_NAME_SIZE];
-#endif
 
   (void) memset((char *)Header, 0, sizeof(*Header));
 
   Header->newsgroup = newsgroup;
 
   if (article > 0) {
-#define CHECK(field,name) if (art->field) Header->field = XtNewString(art->field); else xhdr(newsgroup, article, name, &Header->field);
+
+#define CHECK(field,name) \
+    if (art->field) \
+      Header->field = XtNewString(art->field); \
+    else { \
+      ART_STRUCT_UNLOCK; \
+      xhdr(newsgroup, article, name, &Header->field); \
+      art = artStructGet(newsgroup, article, False); \
+    }
 
     struct article *art = artStructGet(newsgroup, article, False);
 
@@ -353,6 +344,9 @@ static int getHeader(article, Header)
     CHECK(references, "references");
     CHECK(from, "from");
     CHECK(id, "message-id");
+
+    ART_STRUCT_UNLOCK;
+
     xhdr(newsgroup, article, "followup-to", &Header->followupTo);
     xhdr(newsgroup, article, "sender", &Header->sender);
     xhdr(newsgroup, article, "reply-to", &Header->replyTo);
@@ -402,10 +396,8 @@ static int getHeader(article, Header)
   host[0] = '\0';
   host[sizeof(host)-1] = '\0';
 
-#ifndef INN
   path[0] = '\0';
   path[sizeof(path)-1] = '\0';
-#endif
 
 #ifdef	UUCPNAME
   {
@@ -463,15 +455,6 @@ static int getHeader(article, Header)
   if ((! host[0]) && app_resources.hiddenHost && *app_resources.hiddenHost)
     (void) strncpy(host, app_resources.hiddenHost, sizeof(host)-1);
 
-#ifdef INN
-  if (! host[0]) {
-    /* always let values in inn.conf take precedence */
-    char *inn_fromhost = GetConfigValue("fromhost");
-    if (inn_fromhost)
-      (void) strncpy(host, inn_fromhost, sizeof(host)-1);
-  }
-#endif /* INN */
-
 #ifdef HIDDEN_FILE
   if (! host[0])
     if (ptr = getinfofromfile(HIDDEN_FILE))
@@ -486,7 +469,6 @@ static int getHeader(article, Header)
   if (! host[0])
     (void) strcpy(host, real_host);
 
-#ifndef INN
   if (! path[0])
     if ((ptr = getenv("HIDDENPATH")))
       (void) strncpy(path, ptr, sizeof(path)-1);
@@ -497,7 +479,6 @@ static int getHeader(article, Header)
 # endif
   if (! path[0])
     (void) strcpy(path, real_host);
-#endif /* ! INN */
     
   /* If the host name is not a full domain name, put in the domain */
   if (index (host, '.') == NIL(char)) {
@@ -505,7 +486,7 @@ static int getHeader(article, Header)
 
     if (! domain)
       domain = getenv("DOMAIN");
-#ifdef INN
+#ifdef HAVE_CONFIGDATA_H
     if (! domain)
       domain = GetFileConfigValue("domain");
 #endif
@@ -524,7 +505,7 @@ static int getHeader(article, Header)
       return XRN_ERROR;
     }
 
-    (void) strncat(host, domain, sizeof(host) - strlen(host));
+    (void) strncat(host, domain, sizeof(host) - strlen(host) - 1);
   }
 
   Header->host = XtNewString(host);
@@ -532,9 +513,7 @@ static int getHeader(article, Header)
 #ifndef INEWS
   Header->sender_host = XtNewString(sender_host);
 #endif
-#ifndef INN    
   Header->path = XtNewString(path);
-#endif /* INN */
 
   if (app_resources.organization != NIL(char)) {
     Header->organization = XtNewString(app_resources.organization);
@@ -544,10 +523,6 @@ static int getHeader(article, Header)
   } else if ((ptr = getenv ("NEWSORG")) != NIL(char)) {
 #endif
     Header->organization = XtNewString(ptr);
-#ifdef INN
-  } else if ((inn_org = GetConfigValue("organization"))) {
-    Header->organization = XtNewString(inn_org);
-#endif /* INN */    
 #ifdef ORG_FILE
   } else if ((ptr = getinfofromfile(ORG_FILE)) != NULL) {
     Header->organization = XtNewString(ptr);
@@ -811,9 +786,7 @@ static void freeHeader(Header)
 #ifndef INEWS
     FREE(Header->sender_host);
 #endif
-#ifndef INN	
     FREE(Header->path);
-#endif /* INN */	
     FREE(Header->organization);
     return;
 }
@@ -988,7 +961,6 @@ static void buildPath(Header, msg, msg_size, msg_total_size)
     int message_size = *msg_size;
     int message_total_size = *msg_total_size;
 
-#ifndef INN
 #if defined(INEWS) || defined(HIDE_PATH)
     CHECK_SIZE(sizeof("Path: \n") - 1 + strlen(Header->user));
     (void) sprintf(&message[strlen(message)], "Path: %s\n", Header->user);
@@ -998,7 +970,6 @@ static void buildPath(Header, msg, msg_size, msg_total_size)
     (void) sprintf(&message[strlen(message)], "Path: %s!%s\n",
 		   Header->path, Header->user);
 #endif /* INEWS or HIDE_PATH */
-#endif /* INN */
 
     *msg = message;
     *msg_size = message_size;
@@ -1284,6 +1255,45 @@ static unsigned long newsgroupsStatusUnion(newsgroups)
     return(status);
 }
 
+/*
+  Strip empty fields from the header.  Assumes that there is a body,
+  i.e., that the header is terminated by two consecutive blank lines.
+  Modifies the compose text if a header was returned, and returns the
+  buffer containing the text (the original buffer if nothing was
+  changed, or a new buffer after freeing the old one otherwise).  */
+
+static char * stripEmptyFields(char *buffer)
+{
+  char *start, *value, *end;
+  long offset = 0;
+
+  start = buffer;
+  while (*start != '\n') {
+    end = strstr(start, "\n");
+    while (isspace(end[1]) && (end[1] != '\n'))
+      end = strstr(end+1, "\n");
+    value = strstr(start, ":");
+    if (value > end) {
+      /* Broken field has no colon in it.  Don't try to figure it out,
+	 just ignore it. */
+      start = end+1;
+      continue;
+    }
+    for (value++; (value < end) && isspace(*value); value++) /* empty */;
+    if (value == end) {
+      /* empty field! */
+      TextReplace(ComposeText, "", 0,
+		  start-buffer-offset, end-buffer-offset+1);
+      offset += end-start+1;
+    }
+    start = end+1;
+  }
+  if (offset) {
+    XtFree(buffer);
+    buffer = TextGetString(ComposeText);
+  }
+  return buffer;
+}
 
 #define CLEANUP() { XtFree(buffer); XtFree(buffer2); XtFree(buffer3); }
 
@@ -1301,18 +1311,10 @@ void compSendFunction(widget, event, string, count)
     char *ErrMessage;
     int mesg_name = newMesgPaneName();
 
-    /*
-     * I am loathe to use buffers that are 1024 bytes long for the old
-     * from line, new from line and sender line, since they are almost
-     * certainly going to be much shorter than this, but this seems to
-     * be the way things are done elsewhere in this file, so I'll
-     * stick with it.
-     */
     char *ptr, *ptr2;
     int mode, i, j, comma;
     unsigned long newsgroups_status WALL(= 0);
     int tries = 1, saved = 0;
-    int retry_editing = 0;
     int saved_dead = 0;
 
     TextDisableRedisplay(ComposeText);
@@ -1351,6 +1353,7 @@ void compSendFunction(widget, event, string, count)
 	}
 
  	/* Let's be more strict, since inews doesn't see empty headers.  */
+	buffer_size = 0;
  	returnField("Subject:", &buffer, &buffer_size, &buffer_total_size);
 	ptr = buffer + sizeof("Subject:") - 1;
 	while ((*ptr == ' ') || (*ptr == '\t') || (*ptr == '\n'))
@@ -1391,6 +1394,7 @@ void compSendFunction(widget, event, string, count)
 	    CLEANUP(); return;
 	}
 
+	buffer_size = 0;
 	returnField("From:", &buffer, &buffer_size, &buffer_total_size);
 
 	if (! *buffer) {
@@ -1520,6 +1524,7 @@ void compSendFunction(widget, event, string, count)
 	    CLEANUP(); return;
 	}
 
+	buffer_size = 0;
 	returnField("Newsgroups:", &buffer, &buffer_size, &buffer_total_size);
 
 	/* "buffer" now contains the original "Newsgroups" field. */
@@ -1615,7 +1620,7 @@ void compSendFunction(widget, event, string, count)
 	      sprintf(buf, CROSSPOST_CONFIRM_MSG, i) &&
 	      (ChoiceBox(TopLevel, buf, 2, POST_MSG, EDIT_MSG) == 2)) {
 	    if (app_resources.editorCommand)
-	      Call_Editor(True, False);
+	      Call_Editor(True, True);
 	    TextEnableRedisplay(ComposeText);
 	    CLEANUP(); XtFree(buf); return;
 	  }
@@ -1639,7 +1644,7 @@ void compSendFunction(widget, event, string, count)
 		  sprintf(buf, FOLLOWUP_FOLLOWUPTO_CONFIRM_MSG, i, j) &&
 		  (ChoiceBox(TopLevel, buf, 2, POST_MSG, EDIT_MSG) == 2)) {
 		if (app_resources.editorCommand)
-		  Call_Editor(True, False);
+		  Call_Editor(True, True);
 		TextEnableRedisplay(ComposeText);
 		CLEANUP(); XtFree(buf); return;
 	      }
@@ -1652,7 +1657,7 @@ void compSendFunction(widget, event, string, count)
 	      sprintf(buf, FOLLOWUP_CONFIRM_MSG, i);
 	      if (ChoiceBox(TopLevel, buf, 2, POST_MSG, EDIT_MSG) == 2) {
 		if (app_resources.editorCommand)
-		  Call_Editor(True, False);
+		  Call_Editor(True, True);
 		TextEnableRedisplay(ComposeText);
 		CLEANUP(); XtFree(buf); return;
 	      }
@@ -1669,18 +1674,12 @@ void compSendFunction(widget, event, string, count)
 	    CLEANUP(); return;
 	}
 
-#ifndef INN
-	/* The inews in INN adds the Path: header for us, and has its own
-	 * idea of policy.  We don't bother creating it, freeing it, or
-	 * anything else in compose.c that has to do with it
-	 */
 	if (fieldExists("Path:", 0) < 0) {
 	    buffer_size = 0;
 	    *buffer = '\0';
 	    buildPath(&Header, &buffer, &buffer_size, &buffer_total_size);
 	    addField(buffer);
 	}
-#endif /* INN */
 	
 /*	stripField("Message-ID:");  .............we need this !!! */
 	stripField("Relay-Version:");
@@ -1692,9 +1691,9 @@ void compSendFunction(widget, event, string, count)
 	buffer_size = 0;
 	CHECK_SIZE_EXTENDED(buffer, buffer_size, buffer_total_size,
 			    sizeof("X-newsreader: xrn \n") +
-			    sizeof(XRN_VERSION) - 1); /* - 1 because we don't
+			    sizeof(PACKAGE_VERSION) - 1); /* - 1 because we don't
 							 need both nulls */
-	(void) sprintf(buffer, "X-newsreader: xrn %s\n", XRN_VERSION);
+	(void) sprintf(buffer, "X-newsreader: xrn %s\n", PACKAGE_VERSION);
 	addField(buffer);
 #endif
     } else {
@@ -1706,8 +1705,8 @@ void compSendFunction(widget, event, string, count)
 	buffer_size = 0;
 	CHECK_SIZE_EXTENDED(buffer, buffer_size, buffer_total_size,
 			    sizeof("X-mailer: xrn \n") +
-			    sizeof(XRN_VERSION) - 1);
-	(void) sprintf(buffer, "X-mailer: xrn %s\n", XRN_VERSION);
+			    sizeof(PACKAGE_VERSION) - 1);
+	(void) sprintf(buffer, "X-mailer: xrn %s\n", PACKAGE_VERSION);
 	addField(buffer);
 #endif
     }
@@ -1744,6 +1743,9 @@ void compSendFunction(widget, event, string, count)
       return;
     }
 
+    /* Strip empty fields, since some NNTP servers choke on them. */
+    ptr = stripEmptyFields(ptr);
+    
     if ((PostingMode == FOLLOWUP_REPLY) || (PostingMode == POST_MAIL)) {
 	tries = 2;
     }
@@ -1767,7 +1769,6 @@ void compSendFunction(widget, event, string, count)
 		saveDeadLetter(ptr);
 		saved_dead++;
 	    }
-	    retry_editing++;
 	    break;
 	
 	case POST_NOTALLOWED:
@@ -1931,8 +1932,7 @@ getIncludedArticleText()
 
      if (PostingMode == FORWARD) {
 	 int line_size;
-       (void) sprintf(input, FORWARDED_ARTICLE_END_MSG);
-	 line_size = strlen(input);
+	 line_size = strlen(FORWARDED_ARTICLE_END_MSG);
 	 if (prefix_size + line_size > size - cur_size - 1) {
 	     /* See above */
 	     size += MIN(size,REALLOC_MAX);
@@ -1940,7 +1940,7 @@ getIncludedArticleText()
 	 }
 	 (void) strcpy(&text[cur_size], prefix);
 	 cur_size += prefix_size;
-	 (void) strcpy(&text[cur_size], input);
+	 (void) strcpy(&text[cur_size], FORWARDED_ARTICLE_END_MSG);
 	 cur_size += line_size;
      }
 
@@ -2031,7 +2031,7 @@ static void includeHandler(widget, client_data, call_data)
     struct stat sb;
     int num_read, total_read = 0;
 
-    if ((int) client_data == XRNinclude_ABORT) {
+    if ((POINTER_NUM_TYPE) client_data == XRNinclude_ABORT) {
 	/* empty */
     }
     else if (! (file = GetDialogValue(IncludeBox))) {
@@ -2222,24 +2222,30 @@ static int forkpid;
 
 extern int outchannel;
 
-static int catch_sigchld _ARGUMENTS((int));
+static RETSIGTYPE catch_sigchld _ARGUMENTS((int));
 
-static int catch_sigchld(signo)
+static RETSIGTYPE catch_sigchld(signo)
     int signo;
 {
 
     if (signo != SIGCHLD) {
 	/* whoops! */
+#ifdef SIGFUNC_RETURNS
 	return 1;
+#endif
     }
     if (forkpid != wait(0)) {
 	/* whoops! */
+#ifdef SIGFUNC_RETURNS
 	return 1;
+#endif
     }
     (void) signal(SIGCHLD, SIG_DFL);
     editing_status = Completed;
     write(outchannel,"1",1);
+#ifdef SIGFUNC_RETURNS
     return 1;
+#endif
 }
 
 
@@ -2358,7 +2364,7 @@ Call_Editor(
 	    for (i = 3; i < maxdesc; i++) {
 		(void) close(i);
 	    }
-	    (void) execl("/bin/sh", "sh", "-c", buffer, 0);
+	    (void) execl("/bin/sh", "sh", "-c", buffer, NULL);
           (void) fprintf(stderr, ERROR_EXEC_FAILED_MSG, buffer);
 	    (void) _exit(127);
 	}
@@ -2370,7 +2376,7 @@ Call_Editor(
 	    return (-1);
 	}
 	else {
-	    signal(SIGCHLD, (SIG_PF0) catch_sigchld);
+	    signal(SIGCHLD, catch_sigchld);
 	}
 
    return (0);
@@ -2445,12 +2451,12 @@ static int composePane(titleString, header, point)
     XtVaGetValues(TopLevel,
 		  XtNwidth, (XtPointer) &width_val,
 		  XtNheight, (XtPointer) &height_val,
-		  (char *) 0);
+		  (char *) (String)0);
 
     pane = XtVaCreateManagedWidget("pane", panedWidgetClass, ComposeTopLevel,
 				   XtNwidth, (XtArgVal) width_val,
 				   XtNheight, (XtArgVal) height_val,
-				   (char *) 0);
+				   (char *) (String)0);
 
     (void) strcpy(titleStorage, titleString);
 
@@ -2480,7 +2486,7 @@ static int composePane(titleString, header, point)
 
 	XtVaGetValues(ComposeLabel,
 		      XtNheight, (XtPointer) &height_val,
-		      (char *) 0);
+		      (String) 0);
 
 	XawPanedSetMinMax(ComposeLabel, (int) height_val, (int) height_val);
 	XawPanedAllowResize(TEXT_PANE_CHILD(ComposeText), True);
@@ -2743,7 +2749,8 @@ static char *update_headers(Header, first_time, followup, reply)
 	  num_groups >= app_resources.warnings.followup.crossPost)
 	mesgPane(XRN_WARNING, 0, FOLLOWUP_MULTIPLE_NGS_MSG);
       if (app_resources.warnings.followup.followupTo &&
-	  *Header->followupTo && strcmp(Header->followupTo, Header->newsgroups))
+	  *Header->followupTo && strcmp(Header->followupTo, Header->newsgroups) &&
+	  strcmp(Header->followupTo, "poster"))
 	mesgPane(XRN_WARNING, 0, FOLLOWUP_FOLLOWUPTO_MSG);
     }
 
@@ -2856,6 +2863,7 @@ static void followup_or_reply(followup, reply)
 	return;
 
     art = artStructGet(newsgroup, current, False);
+    ART_STRUCT_UNLOCK;
     if (art->file)
       file_cache_file_copy(FileCache, *art->file, &Header.artFile);
 
@@ -2898,6 +2906,8 @@ static void followup_or_reply(followup, reply)
     message_size = strlen(message);
     message_total_size = message_size;
     
+    buildExtraFields(&message, &message_size, &message_total_size);
+
     CHECK_SIZE(1);
     (void) strcat(message, "\n");
 
@@ -2926,7 +2936,7 @@ static void switch_message_type(Header)
     return;
 
   title = followup_or_reply_title(Header, followup, reply);
-  XtVaSetValues(ComposeLabel, XtNlabel, title, 0);
+  XtVaSetValues(ComposeLabel, XtNlabel, title, (String)0);
 
   headers = update_headers(Header, False, followup, reply);
   addField(headers);
@@ -2990,10 +3000,10 @@ void gripe()
     buildReplyTo(&message, &message_size, &message_total_size);
 
     CHECK_SIZE(sizeof("To: \nSubject: GRIPE about XRN \n") - 1 +
-	       sizeof(GRIPES) - 1 + sizeof(XRN_VERSION) - 1);
+	       sizeof(GRIPES) - 1 + sizeof(PACKAGE_VERSION) - 1);
     (void) sprintf(&message[strlen(message)],
 		   "To: %s\nSubject: GRIPE about XRN %s\n",
-		   GRIPES, XRN_VERSION);
+		   GRIPES, PACKAGE_VERSION);
 
     CHECK_SIZE(sizeof(bugTemplate) - 1 + 2);
     (void) strcat(message, "\n");
@@ -3029,6 +3039,7 @@ void forward()
 	return;
 
     art = artStructGet(newsgroup, current, False);
+    ART_STRUCT_UNLOCK;
     if (art->file)
       file_cache_file_copy(FileCache, *art->file, &Header.artFile);
 
@@ -3341,9 +3352,9 @@ void cancelArticle()
     (void) sprintf(&message[strlen(message)], "Control: cancel %s\n",
 		   CancelHeader.id);
 
-    CHECK_SIZE(sizeof("\nCancel message from XRN .\n") + sizeof(XRN_VERSION) - 2);
+    CHECK_SIZE(sizeof("\nCancel message from XRN .\n") + sizeof(PACKAGE_VERSION) - 2);
     (void) sprintf(&message[strlen(message)], "\nCancel message from XRN %s.\n",
-		   XRN_VERSION);
+		   PACKAGE_VERSION);
 
     switch (postArticle(message, XRN_NEWS,&ErrMessage)) {
     case POST_FAILED:
@@ -3623,11 +3634,11 @@ static char *insert_courtesy_tag(message)
   Returns true if all this is OK or false otherwise.
 */
 
-static Boolean check_address _ARGUMENTS((CONST char *));
-static Boolean check_stuff _ARGUMENTS((CONST char *, Boolean));
+static Boolean check_address _ARGUMENTS((const char *));
+static Boolean check_stuff _ARGUMENTS((const char *, Boolean));
 
 static Boolean IsValidAddressField(value_in)
-     CONST char *value_in;
+     const char *value_in;
 {
   char *value, *value_buf, *address, *stuff, *ptr, *end;
   Boolean retval = True;
@@ -3701,11 +3712,11 @@ static Boolean IsValidAddressField(value_in)
 */
 
 static Boolean check_address(address)
-     CONST char *address;
+     const char *address;
 {
   static char special_chars[] = "()<>@,;:\\\"[] ";
   Boolean in_quotes, in_quoted_pair, in_local_part, found_dot;
-  CONST char *ptr;
+  const char *ptr;
 
   in_quotes = in_quoted_pair = found_dot = False;
   in_local_part = True;
@@ -3758,16 +3769,16 @@ static Boolean check_address(address)
 */
 
 static Boolean check_stuff(
-			   _ANSIDECL(CONST char *,	stuff),
+			   _ANSIDECL(const char *,	stuff),
 			   _ANSIDECL(Boolean,		needs_quoting)
 			   )
-     _KNRDECL(CONST char *,	stuff)
+     _KNRDECL(const char *,	stuff)
      _KNRDECL(Boolean,		needs_quoting)
 {
   static char specials[] = "()<>\"";
   static char quoted_specials[] = ".,";
 
-  CONST char *ptr, *end;
+  const char *ptr, *end;
   Boolean is_quoted = False;
 
   end = strchr(stuff, '\0');
