@@ -1,8 +1,4 @@
-
-#if !defined(lint) && !defined(SABER) && !defined(GCC_WALL)
-static char XRNrcsid[] = "$Id: dialogs.c,v 1.23 1998/07/05 14:39:13 jik Exp $";
 /* Modified 2/20/92 dbrooks@osf.org to clean up dialog layout */
-#endif
 
 /*
  * xrn - an X-based NNTP news reader
@@ -217,7 +213,7 @@ static void cbHandler(widget, client_data, call_data)
     XtPointer client_data;
     XtPointer call_data;
 {
-    retVal = (int) client_data;
+    retVal = (POINTER_NUM_TYPE) client_data;
     return;
 }
 
@@ -279,7 +275,7 @@ int ChoiceBox(parent, message, count, va_alist)
   XEvent ev;
   Widget widget;
   XtAppContext app = XtWidgetToApplicationContext(parent);
-  int i;
+  POINTER_NUM_TYPE i;
   va_list args;
 
   dialog_args = (struct DialogArg *) XtMalloc(sizeof(*dialog_args) * count);
@@ -327,7 +323,7 @@ static void passwordHandler(widget, client_data, call_data)
 {
   Widget dialog = XtParent(XtParent(widget));
 
-  password_result = (int) client_data;
+  password_result = (POINTER_NUM_TYPE) client_data;
   if (password_result == XRN_CB_CONTINUE) {
     dialog_password = GetDialogValue(dialog);
     dialog_password = XtNewString(dialog_password);
@@ -349,7 +345,7 @@ String PasswordBox(widget, prompt)
   password_result = -1;
 
   dialog = CreateDialog(TopLevel, prompt, DIALOG_TEXT, args, XtNumber(args));
-  XtVaSetValues(XtNameToWidget(dialog, "dialog.value"), XtNecho, 0, 0);
+  XtVaSetValues(XtNameToWidget(dialog, "dialog.value"), XtNecho, 0, (String)0);
   PopUpDialog(dialog);
 
   while (password_result < 0) {
