@@ -1,7 +1,3 @@
-#if !defined(lint) && !defined(SABER) && !defined(GCC_WALL)
-static char XRNrcsid[] = "$Id: mesg_strings.c,v 1.91 1998/12/30 14:25:53 jik Exp $";
-#endif
-
 /*
  * xrn - an X-based NNTP news reader
  *
@@ -273,6 +269,10 @@ char *message_strings[] = {
     "XRN error: bad button name `%s'.", /* button name */
 /* < NO_SUCH_NG_DELETED > */
     "Newsgroup `%s' does not exist.\n\tIt may have been deleted.", /* newsgroup name */
+/* < GROUP_ACCESS_DENIED > */
+    "Access to newsgroup `%s' denied by server.", /* newsgroup name */
+/* < AUTH_FAILED_RETRY > */
+    "Authentication failed.  Retry?",
 /* < UNKNOWN_FUNC_RESPONSE > */
     "Internal XRN error: unknown response %d from %s in %s.", /* return value, called function, calling function */
 /* < DISPLAYING_LAST_UNREAD > */
@@ -286,7 +286,7 @@ char *message_strings[] = {
 /* < NO_NG_SPECIFIED > */
     "No newsgroup name specified.",
 /* < NO_SUCH_NG > */
-    "Newsgroup `%s' does not exist.", /* newsgroup name */
+    "Newsgroup `%s' does not exist or you can't access it.", /* newsgroup name */
 /* < NO_PREV_NG > */
     "No previous newsgroup.",
 /* < NO_GROUPS_SELECTED > */
@@ -665,13 +665,13 @@ char *message_strings[] = {
  ### may be the following messages shouldn't translate ###
  */
 /* < REPLY_YOU_WRITE > */
-    "In article %s,\n you write:\n", /* messageid */
+    "You write:\n", /* messageid */
 /* < FORWARDED_ARTIKEL > */
     "\n------ Forwarded Article %s\n------ From %s\n\n", /* messageid , author */
 /* < FORWARDED_ARTICLE_END > */
     "\n------ End of Forwarded Article\n",
 /* < FOLLOWUP_AUTHOR_WRITE > */
-    "In article %s,\n %s writes:\n", /* messageid , author */
+    "%.0s%s writes:\n", /* messageid , author */
 /* #### end may be not translate #### */
 /* < NEWSGROUPS_INDEX > */
     "%6s %7s %*s %4d article%1.1s +%6d old",
@@ -792,6 +792,10 @@ char *message_strings[] = {
     "Erreur XRN : nom de bouton incorrect `%s'.", /* button name */
 /* < NO_SUCH_NG_DELETED > */
     "Le groupe de news `%s' n'existe pas.\n\tIl peut avoir t supprim.", /* newsgroup name */
+/* < GROUP_ACCESS_DENIED > */
+    "Access to newsgroup `%s' denied by server.", /* newsgroup name */
+/* < AUTH_FAILED_RETRY > */
+    "Authentication failed.  Retry?",
 /* < UNKNOWN_FUNC_RESPONSE > */
     "Internal XRN error: unknown response %d from %s in %s.", /* return value, called function, calling function */
 /* < DISPLAYING_LAST_UNREAD > */
@@ -805,7 +809,7 @@ char *message_strings[] = {
 /* < NO_NG_SPECIFIED > */
     "Pas de nom de groupe de news spcifi.",
 /* < NO_SUCH_NG > */
-    "Le groupe de news `%s' n'existe pas.", /* newsgroup name */
+    "Newsgroup `%s' does not exist or you can't access it.", /* newsgroup name */
 /* < NO_PREV_NG > */
     "Pas de groupe de news prcdent",
 /* < NO_GROUPS_SELECTED > */
@@ -1184,13 +1188,13 @@ char *message_strings[] = {
  ### les messages ci-dessous ne devraient pas tre traduits ###
  */
 /* < REPLY_YOU_WRITE > */
-    "In article %s,\n you write:\n", /* messageid */
+    "You write:\n", /* messageid */
 /* < FORWARDED_ARTIKEL > */
     "\n------ Forwarded Article %s\n------ From %s\n\n", /* messageid , author */
 /* < FORWARDED_ARTICLE_END > */
     "\n------ End of Forwarded Article\n",
 /* < FOLLOWUP_AUTHOR_WRITE > */
-    "In article %s,\n %s writes:\n", /* messageid , author */
+    "%.0s%s writes:\n", /* messageid , author */
 /* #### fin des messages  ne pas traduire #### */
 /* < NEWSGROUPS_INDEX > */
     "%9s %13s %*s %4d article%1.1s +%5d ancien%1.1s",
@@ -1330,6 +1334,10 @@ char *message_strings[] = {
     "XRN Fehler: Falscher Knopf-Name `%s'.", /* button Name */
 /* < NO_SUCH_NG_DELETED > */
     "Newsgruppe `%s' existiert nicht.\n\tM\366glicherweise wurde sie entfernt.", /* Newsgruppe Name */
+/* < GROUP_ACCESS_DENIED > */
+    "Access to newsgroup `%s' denied by server.", /* newsgroup name */
+/* < AUTH_FAILED_RETRY > */
+    "Authentication failed.  Retry?",
 /* < UNKNOWN_FUNC_RESPONSE > */
     "Interner XRN Fehler: unbekannte R\374ckmeldung %d von %s in %s.", /* return value, called function, calling function */
 /* < DISPLAYING_LAST_UNREAD > */
@@ -1343,7 +1351,7 @@ char *message_strings[] = {
 /* < NO_NG_SPECIFIED > */
     "Keine Newsgruppe angegeben.",
 /* < NO_SUCH_NG > */
-    "Newsgruppe `%s' existiert nicht.", /* Newsgruppe Name */
+    "Newsgroup `%s' does not exist or you can't access it.", /* newsgroup name */
 /* < NO_PREV_NG > */
     "Keine vorhergehende Newsgruppe.",
 /* < NO_GROUPS_SELECTED > */
@@ -1722,13 +1730,13 @@ char *message_strings[] = {
  ### Die folgenden Texte sollten evtl. nicht uebersetzt werden ###
  */
 /* < REPLY_YOU_WRITE > */
-    "In article %s,\n you write:\n", /* messageid */
+    "You write:\n", /* messageid */
 /* < FORWARDED_ARTIKEL > */
     "\n------ Forwarded Article %s\n------ From %s\n\n", /* messageid , author */
 /* < FORWARDED_ARTICLE_END > */
     "\n------ End of Forwarded Article\n",
 /* < FOLLOWUP_AUTHOR_WRITE > */
-    "In article %s,\n %s writes:\n", /* messageid , author */
+    "%.0s%s writes:\n", /* messageid , author */
 /* #### Ende des evtl. nicht uebersetzen #### */
 /* < NEWSGROUPS_INDEX > */
     "%4s %10s %*s %4d Artikel%1.1s +%6d alt",
