@@ -221,6 +221,7 @@ void ngReadFunction(widget, event, string, count)
 	    switchToArticleMode();
 	}
 	else {
+	    exitNewsgroup();
 	    if (status == XRN_NOUNREAD)
 		mesgPane(XRN_INFO, 0, PROBABLY_KILLED_MSG, name);
 	    else if (status == BAD_GROUP)
@@ -425,7 +426,7 @@ static void subscribeHandler(widget, client_data, call_data)
 
     TextDisableRedisplay(NewsgroupText);
 
-    switch ((int) client_data) {
+    switch ((POINTER_NUM_TYPE) client_data) {
     case XRNsub_LASTGROUP:
 	if (LastGroup && *LastGroup) {
 	    if ((ret = enterNewsgroup(LastGroup, ENTER_UNSUBBED))
@@ -434,12 +435,14 @@ static void subscribeHandler(widget, client_data, call_data)
 		goto done;
 	    }
 	    else if (ret == GOOD_GROUP) {
-		subscribe();
+	      if (subscribe()) {
 		CurrentIndexGroup = XtRealloc(CurrentIndexGroup,
 					      strlen(LastGroup) + 1);
 		(void) strcpy(CurrentIndexGroup, LastGroup);
 		updateNewsgroupMode(True, False);
 		exitNewsgroup();
+	      }
+	      
 	    }
 	    else {
 		mesgPane(XRN_SERIOUS, 0, UNKNOWN_FUNC_RESPONSE_MSG, ret,
@@ -567,7 +570,7 @@ static void gotoHandler(widget, client_data, call_data)
     inCommand = 1;
     xrnBusyCursor();
     TextUnsetSelection(NewsgroupText);
-    if ((int) client_data == XRNgoto_GOTO) {
+    if ((POINTER_NUM_TYPE) client_data == XRNgoto_GOTO) {
 	name = GetDialogValue(GotoNewsgroupBox);
 	if (name[0] == '\0') {
 	    mesgPane(XRN_INFO, 0, NO_NG_SPECIFIED_MSG);
@@ -598,6 +601,7 @@ static void gotoHandler(widget, client_data, call_data)
 		  mesgPane(XRN_SERIOUS, 0, NO_SUCH_NG_MSG, name);
 	     }
 	     else if (ret == XRN_NOMORE) {
+		  exitNewsgroup();
 		  mesgPane(XRN_SERIOUS, 0, NO_ARTICLES_MSG, name);
 	     }
 	     else {
@@ -755,6 +759,7 @@ void ngPrevGroupFunction(widget, event, string, count)
 	    mesgPane(XRN_SERIOUS, 0, NO_SUCH_NG_DELETED_MSG, LastGroup);
 	}
 	else if (ret == XRN_NOMORE) {
+	    exitNewsgroup();
 	    mesgPane(XRN_SERIOUS, 0, NO_ARTICLES_MSG, LastGroup);
 	}
 	else {
@@ -1148,7 +1153,10 @@ void switchToNewsgroupMode(
 
     /* update the newsgroup mode windows */
     updateNewsgroupMode(True, skip_last);
-    
+
+    /* Set up the rescan timer, if automatic rescanning is configured. */
+    addTimeOut();
+
     return;
 }
 
@@ -1191,6 +1199,10 @@ static void autoRescan _ARGUMENTS((XtPointer, XtIntervalId *));
 
 void addTimeOut()
 {
+#ifdef DEBUG
+  fprintf(stderr, "addTimeOut()\n");
+#endif
+
     if (CurrentMode != NEWSGROUP_MODE) {
 	return;
     }
@@ -1203,8 +1215,6 @@ void addTimeOut()
     if (TimeOut) {
 	return;
     }
-    /* handle race conditions??? */
-    TimeOut = 1;
 
     TimeOut = XtAppAddTimeOut(TopContext,
 			      app_resources.rescanTime * 1000, autoRescan, 0);
@@ -1214,19 +1224,18 @@ void addTimeOut()
 
 void removeTimeOut()
 {
-    XtIntervalId temp;
+#ifdef DEBUG
+    fprintf(stderr, "removeTimeOut()\n");
+#endif
 
     if (CurrentMode != NEWSGROUP_MODE) {
 	return;
     }
 
-    /* handle race conditions??? */
-    temp = TimeOut;
-    TimeOut = 0;
-
     /* do not allow recursive timeouts */
-    if (temp) {
-	XtRemoveTimeOut(temp);
+    if (TimeOut) {
+	XtRemoveTimeOut(TimeOut);
+	TimeOut = 0;
     }
     return;
 }
@@ -1239,6 +1248,10 @@ static void autoRescan(data, id)
     String params[1];
     Cardinal num_params = 1;
 
+#ifdef DEBUG
+    fprintf(stderr, "autoRescan(...)\n");
+#endif
+
     params[0] = "nonewgroups";
 
     if (CurrentMode != NEWSGROUP_MODE) {
