@@ -1,8 +1,3 @@
-
-#if !defined(lint) && !defined(SABER) && !defined(GCC_WALL)
-static char XRNrcsid[] = "$Id: refile.c,v 1.16 1999/08/30 12:13:14 jik Exp $";
-#endif
-
 /*
  * xrn - an X-based NNTP news reader
  *
@@ -42,7 +37,7 @@ static char XRNrcsid[] = "$Id: refile.c,v 1.16 1999/08/30 12:13:14 jik Exp $";
 #else
 #include <sys/file.h>
 #endif
-#if defined(_XOPEN_SOURCE)
+#if STDC_HEADERS
 #include <stdlib.h>
 #else
     char *getenv();
@@ -233,7 +228,7 @@ int RMAILrefile(fullpath, folder, artfile, pos)
 	return(0);
     }
     /* Format the header */
-    fprintf (fp, "\014\n1,,\n");
+    fprintf (fp, "\f\n1,,\n");
     /* insert from 0 to pos (from getarticle) for the header */
     n = 0;
     while ((rv = read (artfd, msg, n + 512 > pos ? pos - n : 512)) > 0) {
