@@ -1,8 +1,3 @@
-
-#if !defined(lint) && !defined(SABER) && !defined(GCC_WALL)
-static char XRNrcsid[] = "$Id: server.c,v 1.153 1999/01/03 16:02:26 jik Exp $";
-#endif
-
 /*
  * xrn - an X-based NNTP news reader
  *
@@ -33,6 +28,10 @@ static char XRNrcsid[] = "$Id: server.c,v 1.153 1999/01/03 16:02:26 jik Exp $";
  *
  */
 
+#include <stdio.h>
+#include <netinet/in.h>
+#include <netdb.h>
+
 #include "copyright.h"
 #include "config.h"
 #include "utils.h"
@@ -40,6 +39,7 @@ static char XRNrcsid[] = "$Id: server.c,v 1.153 1999/01/03 16:02:26 jik Exp $";
 #include <ctype.h>
 #include <assert.h>
 #include <errno.h>
+#include <time.h>
 #include "avl.h"
 #include "news.h"
 #include "artstruct.h"
@@ -78,6 +78,8 @@ static struct newsgroup *currentNewsgroup = 0;
 #define SETNEWSGROUP(n) (((n) == currentNewsgroup) ? 0 : \
 			 getgroup((n), 0, 0, 0, True))
 
+static void get_article_headers(struct newsgroup *, art_num);
+
 /*
   Get a line of data from the server.
 
@@ -172,6 +174,11 @@ static int wants_user_pass_authentication()
   return(! strncasecmp(ptr, "user/pass", sizeof("user/pass")-1));
 }
 
+/*
+  Cache the password so the user only has to be prompted once.
+*/
+static char *authinfo_password = NULL;
+
 /*
   This function assumes that wants_user_pass_authentication() has
   already been called and returned True.  I.e., don't call this
@@ -248,7 +255,12 @@ static int user_pass_authentication()
     user = XtNewString(user);
 
   if (! pass)
-    pass = PasswordBox(TopLevel, NNTP_PASSWORD_MSG);
+    if (authinfo_password)
+      pass = XtNewString(authinfo_password);
+    else {
+      if ((pass = PasswordBox(TopLevel, NNTP_PASSWORD_MSG)))
+	authinfo_password = XtNewString(pass);
+    }
   else
     pass = XtNewString(pass);
 
@@ -269,7 +281,13 @@ static int user_pass_authentication()
   put_server(cmdbuf);
   response = get_data_from_server(True);
   if (*response != CHAR_OK) {
-    retval = -1;
+    if (authinfo_password) {
+      XtFree(authinfo_password);
+      authinfo_password = NULL;
+      retval = 1;
+    }
+    else
+      retval = -1;
     goto done;
   }
 
@@ -283,6 +301,12 @@ done:
 }
 
 
+/*
+  Returns 0 on successful authentication, <0 if authentication failed
+  and can't be retried, or >0 if authentication failed and can be
+  retried.
+*/
+  
 static int authenticate _ARGUMENTS((void));
 
 static int
@@ -336,19 +360,34 @@ authenticate() {
 
     sprintf(cmdbuf, app_resources.authenticatorCommand,
 	    app_resources.authenticator);
-    return (system(cmdbuf));
+    /* We have to assume that authentication can't be retried if this
+       fails, because there's no way for the authenticator command to
+       communicate to us whether it's safe to retry. */
+    return (system(cmdbuf) ? -1 : 0);
 }
 
 static int check_authentication _ARGUMENTS((char *, char **));
 
+Boolean authentication_failure = False;
+
 static int
 check_authentication(command, response)
     char *command;   /* command to resend           */
     char **response; /* response from the command   */
 {
+  int ret;
+
+  authentication_failure = False;
   if (STREQN(*response, "480 ", 4)) {
-    if (authenticate()) {
-      *response = "502 Authentication failed";
+    if (((ret = authenticate()) < 0) ||
+	((ret > 0) && ! ehErrorRetryXRN(AUTH_FAILED_RETRY_MSG, False))) {
+      if (atoi(*response) != 502)
+	*response = "502 Authentication failed";
+      authentication_failure = True;
+    }
+    else if (ret > 0) {
+      ServerDown = True;
+      return(0);
     } else {
       put_server(command);
       *response = get_data_from_server(True);
@@ -377,14 +416,22 @@ static void check_server_response(command, response)
      *   since the error number (503) is used for more than just
      *   timeout
      *
-     *   Message is:
-     *     503 Timeout ...
+     *   Any response with error number 503 containing the string
+     *   "imeout" is treated as a timeout message.  We search for just
+     *   "imeout" rather than "timeout" so that "Timeout" is also
+     *   valid.  We also search for "Time Out".
+     *
+     *   Some servers return code 205 instead of 503, so we check that
+     *   as well.
      */
+  int rcode;
 
     if (check_authentication(command, response))
       return;
 
-    if (ServerDown || STREQN(*response, "503 Timeout", 11)) {
+    if (ServerDown ||
+	((((rcode = atoi(*response)) == 503) || rcode == 205) &&
+	 (strstr(*response, "Time Out") || strstr(*response, "imeout")))) {
 
 	mesgPane(XRN_SERIOUS, 0, LOST_CONNECT_ATTEMPT_RE_MSG);
 	start_server();
@@ -415,7 +462,9 @@ static void check_server_response(command, response)
 
 /*
   Fetch an article from the server into the "base_file" field of the
-  article structure for the article.
+  article structure for the article, unless the "fetch" argument is
+  false, in which case it will only return success if the article has
+  already previously been fetched.
 
   If the article was fetched successfully, returns a positive number
   and fills in the pointer to the fetched cache file structure.
@@ -430,12 +479,18 @@ static void check_server_response(command, response)
 
   The returned cache file is locked until unlocked by the caller. */
 static int get_base_article _ARGUMENTS((struct newsgroup *, art_num,
-					file_cache_file **));
+					file_cache_file **, Boolean));
 
-static int get_base_article(newsgroup, artnum, ret_cache_file)
-     struct newsgroup *newsgroup;
-     art_num artnum;
-     file_cache_file **ret_cache_file;
+static int get_base_article(
+			    _ANSIDECL(struct newsgroup *,	newsgroup),
+			    _ANSIDECL(art_num, 			artnum),
+			    _ANSIDECL(file_cache_file **,	ret_cache_file),
+			    _ANSIDECL(Boolean,			fetch)
+			    )
+     _KNRDECL(struct newsgroup *,	newsgroup)
+     _KNRDECL(art_num,			artnum)
+     _KNRDECL(file_cache_file **,	ret_cache_file)
+     _KNRDECL(Boolean,			fetch)
 {
   struct article *art;
   long start_time, end_time;
@@ -450,9 +505,15 @@ static int get_base_article(newsgroup, artnum, ret_cache_file)
   if (art->base_file && *art->base_file) {
     file_cache_file_lock(FileCache, *art->base_file);
     *ret_cache_file = art->base_file;
+    artStructSet(newsgroup, &art);
     return 1;
   }
 
+  if (! fetch) {
+    artStructSet(newsgroup, &art);
+    return 0;
+  }
+
   CLEAR_BASE_FILE(art);
 
   if (SETNEWSGROUP(newsgroup)) {
@@ -466,17 +527,6 @@ static int get_base_article(newsgroup, artnum, ret_cache_file)
 
   start_time = time(0);
 
-  (void) sprintf(command, "ARTICLE %ld", artnum);
-  put_server(command);
-  message = get_data_from_server(True);
-
-  check_server_response(command, &message);
-
-  if (*message != CHAR_OK) {
-    artStructSet(newsgroup, &art);
-    return 0;
-  }
-
   cache_file = (file_cache_file *) XtMalloc(sizeof(*cache_file));
 
   if (! (fp = file_cache_file_open(FileCache, cache_file))) {
@@ -485,11 +535,24 @@ static int get_base_article(newsgroup, artnum, ret_cache_file)
     FREE(cache_file);
     artStructSet(newsgroup, &art);
     if (ehErrorRetryXRN(error_buffer, True))
-      return get_base_article(newsgroup, artnum, ret_cache_file);
+      return get_base_article(newsgroup, artnum, ret_cache_file, True);
   }
 
   do_chmod(fp, file_cache_file_name(FileCache, *cache_file), 0600);
 
+  (void) sprintf(command, "ARTICLE %ld", artnum);
+  put_server(command);
+  message = get_data_from_server(True);
+
+  check_server_response(command, &message);
+
+  if (*message != CHAR_OK) {
+    artStructSet(newsgroup, &art);
+    file_cache_file_destroy(FileCache, *cache_file);
+    FREE(cache_file);
+    return 0;
+  }
+
   while (1) {
     line = get_data_from_server(False);
 
@@ -535,7 +598,7 @@ static int get_base_article(newsgroup, artnum, ret_cache_file)
       FREE(cache_file);
       artStructSet(newsgroup, &art);
       if (file_cache_free_space(FileCache, 1))
-	return get_base_article(newsgroup, artnum, ret_cache_file);
+	return get_base_article(newsgroup, artnum, ret_cache_file, True);
       return -1;
     }
   }
@@ -580,13 +643,6 @@ static int get_base_article(newsgroup, artnum, ret_cache_file)
  *
  *   returns:  the cache file (locked) that the article is stored in
  *             or NIL(char) if the article is not avaiable
- *
- * MODIFIES THE ARTICLE STRUCTURE FOR THE ARTICLE BEING RETRIEVED.
- * Therefore, the caller should not have the article structure
- * allocated (i.e., returned by artStructGet()) and expect the
- * allocated structure to be valid after the call to getarticle().  If
- * the caller *does* have the structure allocated, he should
- * re-retrieve it after calling getarticle().
  */
 file_cache_file *getarticle(newsgroup, artnumber, retposition, flags)
      struct newsgroup *newsgroup;
@@ -598,7 +654,7 @@ file_cache_file *getarticle(newsgroup, artnumber, retposition, flags)
   file_cache_file *base_cache_file, *cache_file;
   int ret;
   FILE *basefp, *articlefp;
-  static char *buf = NULL;
+  static char *buf = NULL, *buf_base;
   static int buf_size = 0;
   char *buf_ptr, *ptr;
   int byteCount = 0, lineCount = 0, error = 0;
@@ -610,7 +666,7 @@ file_cache_file *getarticle(newsgroup, artnumber, retposition, flags)
   if (SETNEWSGROUP(newsgroup))
     return NULL;
 
-  if ((ret = get_base_article(newsgroup, artnumber, &base_cache_file)) < 0) {
+  if ((ret = get_base_article(newsgroup, artnumber, &base_cache_file, True)) < 0) {
   cache_error:
     sprintf(error_buffer, FILE_CACHE_OPEN_MSG, file_cache_dir_get(FileCache),
 	    errmsg(errno));
@@ -640,7 +696,7 @@ file_cache_file *getarticle(newsgroup, artnumber, retposition, flags)
   do_chmod(articlefp, file_cache_file_name(FileCache, *cache_file), 0600);
 
   if (! buf) {
-    buf = XtMalloc(BUFFER_SIZE);
+    buf_base = buf = XtMalloc(BUFFER_SIZE);
     buf_size = BUFFER_SIZE;
   }
 
@@ -650,7 +706,7 @@ file_cache_file *getarticle(newsgroup, artnumber, retposition, flags)
     if ((strlen(buf_ptr) == buf_size - (buf_ptr - buf) - 1) &&
 	(buf_ptr[buf_size - (buf_ptr - buf) - 2] != '\n')) {
       buf_size *= 2;
-      buf = XtRealloc(buf, buf_size);
+      buf_base = buf = XtRealloc(buf, buf_size);
       buf_ptr = &buf[strlen(buf)];
       continue;
     }
@@ -720,7 +776,7 @@ file_cache_file *getarticle(newsgroup, artnumber, retposition, flags)
 #endif /* XLATE */
 
     /* handle ^L (poorly?) */
-    if (found_sep && (flags & PAGEBREAKS) && (*buf == '\014')) {
+    if (found_sep && (flags & PAGEBREAKS) && (*buf == '\f')) {
       int i, lines;
       lines = server_page_height;
       lines -= lineCount % lines;
@@ -758,6 +814,7 @@ file_cache_file *getarticle(newsgroup, artnumber, retposition, flags)
 
     byteCount += strlen(buf);
     lineCount++;
+    buf = buf_base;
   }
 
   (void) fclose(basefp);
@@ -825,28 +882,37 @@ int getgroup(
 	check_server_response(command, &message);
     
 	if (*message != CHAR_OK) {
-	    if (atoi(message) != ERR_NOGROUP) {
-	      char *mybuf = XtMalloc(strlen(ERROR_REQUEST_FAILED_MSG) +
-				     strlen(command) +
-				     strlen(message));
-	      (void) sprintf(mybuf, ERROR_REQUEST_FAILED_MSG,
-			     command, message);
-	      ehErrorExitXRN(mybuf);
-	    }
+	  int code = atoi(message);
+
+	  if ((code == ERR_ACCESS) && ! authentication_failure) {
 	    if (display_error)
-		mesgPane(XRN_SERIOUS, 0, NO_SUCH_NG_DELETED_MSG,
-			 newsgroup->name);
+	      mesgPane(XRN_SERIOUS, 0, GROUP_ACCESS_DENIED_MSG,
+		       newsgroup->name);
+	  }
+	  else if (code == ERR_NOGROUP) {
+	    if (display_error)
+	      mesgPane(XRN_SERIOUS, 0, NO_SUCH_NG_DELETED_MSG,
+		       newsgroup->name);
+	  }
+	  else {
+	    char *mybuf = XtMalloc(strlen(ERROR_REQUEST_FAILED_MSG) +
+				   strlen(command) +
+				   strlen(message));
+	    (void) sprintf(mybuf, ERROR_REQUEST_FAILED_MSG,
+			   command, message);
+	    ehErrorExitXRN(mybuf);
+	  }
 
-	    /* remove the group from active use ??? */
+	  /* remove the group from active use ??? */
 
-	    if (number)
-	      *number = 0;
-	    if (first)
-	      *first = 0;
-	    if (last)
-	      *last = 0;
+	  if (number)
+	    *number = 0;
+	  if (first)
+	    *first = 0;
+	  if (last)
+	    *last = 0;
 
-	    return(NO_GROUP);
+	  return(NO_GROUP);
 	}
 
 	currentNewsgroup = newsgroup;
@@ -904,10 +970,13 @@ char *newgroupsDate()
 	clock = time(0);
 	curtime = gmtime(&clock);
 	assert(curtime);
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wformat-overflow" /* GCC is confused */
 	(void) sprintf(date_buf, "%02d%02d%02d %02d%02d%02d GMT",
 		       curtime->tm_year % 100, curtime->tm_mon + 1,
 		       curtime->tm_mday, curtime->tm_hour, curtime->tm_min,
 		       curtime->tm_sec);
+#pragma GCC diagnostic pop
     }
 
     return date_buf;
@@ -1063,6 +1132,19 @@ int parse_active_line(
   art_num first, last;
   struct newsgroup *newsgroup;
   int ret;
+  /* We create the scanf_format dynamically so that we can put a field
+     width into it for the "type" field. */
+  static char *scanf_format = NULL;
+
+#define SCANF_FORMAT_FORMAT " %%ld %%ld %%%ds"
+  if (! scanf_format) {
+    /* Boy, this is astoundingly paranoid. */
+    assert(BUFFER_SIZE < 1000000000);
+    scanf_format = XtMalloc(sizeof(SCANF_FORMAT_FORMAT) + 10);
+    (void) sprintf(scanf_format, SCANF_FORMAT_FORMAT, BUFFER_SIZE);
+    memset(type, 0, sizeof(type));
+  }
+#undef SCANF_FORMAT_FORMAT
 
   /* server returns: group last first y/m/x/j/=otherGroup */
 
@@ -1073,8 +1155,12 @@ int parse_active_line(
     return ACTIVE_BOGUS;
   *ptr2 = '\0';
   group = ptr;
-  if (sscanf(ptr2 + 1, " %ld %ld %s", &last, &first, type) != 3)
+  type[sizeof(type)-1] = '\0';
+  if (sscanf(ptr2 + 1, scanf_format, &last, &first, type) != 3)
     return ACTIVE_BOGUS;
+  /* I don't know how to deal with a line that is so long that it doesn't
+     fit in the type buffer.  This really should never happen! */
+  assert(! type[sizeof(type)-1]);
 
   switch (type[0]) {
   case 'x':
@@ -1299,7 +1385,7 @@ void getactive(
         message = get_data_from_server(True);
 	
 	/* the list is ended by a '.' at the beginning of a line */
-	if (*message == '.') {
+	if (message[0] == '.' && ! message[1]) {
 	    break;
 	}
 
@@ -1329,7 +1415,7 @@ void getactive(
     else
       active_read++;
 
-    checkNewsrcSize(ActiveGroupsCount);
+    CHECKNEWSRCSIZE(ActiveGroupsCount);
 #ifndef FIXED_ACTIVE_FILE
     badActiveFileCheck();
 #endif
@@ -1413,6 +1499,7 @@ void start_server()
 
     while (1) {
       response = server_init(server);
+
       if (response == OK_NOPOST)
 	PostingAllowed = False;
       else if (response >= 0)
@@ -1448,6 +1535,17 @@ void stop_server()
 }
 
 
+/* Which header fields can't be fetched with XHDR? */
+#ifdef NO_XHDR_NEWSGROUPS
+static fetch_flag_t no_xhdr_fields = FETCH_NEWSGROUPS;
+#else
+static fetch_flag_t no_xhdr_fields = 0;
+#endif
+
+/* Which header fields *can* be fetched with XHDR? */
+static fetch_flag_t xhdr_fields = 0;
+
+
 /*
  * Get a list of lines for a particular field for the specified group in
  * the range 'first' to 'last'.
@@ -1456,6 +1554,10 @@ void stop_server()
  * number and a string and returns a new, allocated string to actually
  * assign to the field.
  *
+ * "field_bit", is the prefetch field bit, if any, for this field.  It
+ * is used to keep track of which fields the server can't handle XHDR
+ * requests for.
+ * 
  * "offset" is the offset of the field's pointer in a article
  * structure.
  * 
@@ -1464,7 +1566,7 @@ void stop_server()
 typedef char * (*_fixfunction) _ARGUMENTS((struct newsgroup *, art_num, char *));
 
 static Boolean getlist _ARGUMENTS((struct newsgroup *, art_num, art_num,
-				   Boolean, int *, char *, unsigned,
+				   Boolean, int *, char *, fetch_flag_t, unsigned,
 				   _fixfunction, unsigned, Boolean));
 
 static Boolean getlist(
@@ -1474,6 +1576,7 @@ static Boolean getlist(
 		       _ANSIDECL(Boolean,		unreadonly),
 		       _ANSIDECL(int *,			max),
 		       _ANSIDECL(char *,		field),
+		       _ANSIDECL(fetch_flag_t,		field_bit),
 		       _ANSIDECL(unsigned,		offset),
 		       _ANSIDECL(_fixfunction, 		fixfunction),
 		       _ANSIDECL(unsigned,		fixed_offset),
@@ -1485,6 +1588,7 @@ static Boolean getlist(
      _KNRDECL(Boolean,			unreadonly)
      _KNRDECL(int *,			max)
      _KNRDECL(char *,			field)
+     _KNRDECL(fetch_flag_t,		field_bit)
      _KNRDECL(unsigned,			offset)
      _KNRDECL(_fixfunction, 		fixfunction)
      _KNRDECL(unsigned,			fixed_offset)
@@ -1497,6 +1601,7 @@ static Boolean getlist(
   int count = 0;
   struct article *art;
   int pane_name = newMesgPaneName();
+  Boolean trying_again = False;
 
   if (SETNEWSGROUP(newsgroup))
     return True;
@@ -1512,117 +1617,180 @@ static Boolean getlist(
 	(unreadonly && IS_READ(art)) ||
 	IS_UNAVAIL(art)) {
       first++;
+      ART_STRUCT_UNLOCK;
       continue;
     }
+    ART_STRUCT_UNLOCK;
 
     for (sub_count = 1, last = first + 1; last <= artlast; last++) {
       art = artStructGet(newsgroup, last, False);
       if (((offset != (unsigned)-1) && *(char **)((char *) art + offset)) ||
 	  (fixfunction && *(char **)((char *) art + fixed_offset)) ||
 	  (unreadonly && IS_READ(art)) ||
-	  (max && ((count + sub_count) >= *max)))
+	  (max && ((count + sub_count) >= *max))) {
+	ART_STRUCT_UNLOCK;
 	break;
+      }
       if (max && IS_AVAIL(art))
 	sub_count++;
+      ART_STRUCT_UNLOCK;
     }
     last--;
 
-    (void) sprintf(command, "XHDR %s %ld-%ld", field, first, last);
-    put_server(command);
-    message = get_data_from_server(True);
-
-    check_server_response(command, &message);
-
-    /* check for errors */
-    if (*message != CHAR_OK) {
-      mesgPane(XRN_SERIOUS, pane_name, XHDR_ERROR_MSG);
-      return True;
+  try_again:
+    if (trying_again || (newsgroup->fetch & no_xhdr_fields)) {
+      for (number = first; number <= last; number++) {
+	art = artStructGet(newsgroup, number, False);
+	if (IS_UNAVAIL(art) || (unreadonly && IS_READ(art))) {
+	  ART_STRUCT_UNLOCK;
+	  continue;
+	}
+	ART_STRUCT_UNLOCK;
+	get_article_headers(newsgroup, number);
+	art = artStructGet(newsgroup, number, True);
+	if (art->headers && avl_lookup(art->headers, field, &line)) {
+	  if (trying_again)
+	    no_xhdr_fields |= field_bit;
+	  if (offset != (unsigned)-1)
+	    *(char **)((char *)art + offset) =
+	      XtNewString(line);
+	  if (fixfunction)
+	    *(char **)((char *) art + fixed_offset) =
+	      (*fixfunction)(newsgroup, number, line);
+	}
+	artStructSet(newsgroup, &art);
+      }
     }
-
-    for(;;) {
+    else {
+      (void) sprintf(command, "XHDR %s %ld-%ld", field, first, last);
+      put_server(command);
       message = get_data_from_server(True);
-      if (*message == '.') {
-	break;
+
+      check_server_response(command, &message);
+
+      /* check for errors */
+      if (*message != CHAR_OK) {
+	if (field_bit) {
+	  no_xhdr_fields |= field_bit;
+	  goto try_again;
+	}
+	mesgPane(XRN_SERIOUS, pane_name, XHDR_ERROR_MSG);
+	return True;
       }
 
-      /*
-	The brilliant folks at Microsoft have decided that it's OK for
-	XHDR to return multi-line header fields as multiple lines in
-	the XHDR response, even though that's never been done by any
-	other NNTP server and it's different from what XOVER does.
-	It's really annoying how they invent standards like this.  For
-	the time being, I'm going to assume that someone is going to
-	show them the error of their ways and get them to fix their
-	server, so I'm going to ignore the bogus output for now
-	instead of actually trying to handle it.  If they manage to
-	browbeat the world into doing things their way, as they so
-	often do, I'll consider at some point in the future adding
-	support for multi-line XHDR field response.  Grr.
-	 - jik 12/16/97
-      */
-      if (isspace((unsigned char)*message))
-	continue;
+      for(;;) {
+	message = get_data_from_server(True);
+	if (*message == '.') {
+	  break;
+	}
 
-      count++;
-
-      /*
-       * message is of the form:
-       *
-       *    Number value
-       *
-       * must get the number since not all articles will be returned
-       */
-
-      number = atol(message);
-      line = index(message, ' ');
-      if (! (number && line)) {
-	mesgPane(XRN_SERIOUS, pane_name, MALFORMED_XHDR_RESPONSE_MSG,
-		 command, message);
-	goto next_iteration;
-      }
-
-      /*
-	Strip leading and trailing spaces.
+	/*
+	  The brilliant folks at Microsoft have decided that it's OK for
+	  XHDR to return multi-line header fields as multiple lines in
+	  the XHDR response, even though that's never been done by any
+	  other NNTP server and it's different from what XOVER does.
+	  It's really annoying how they invent standards like this.  For
+	  the time being, I'm going to assume that someone is going to
+	  show them the error of their ways and get them to fix their
+	  server, so I'm going to ignore the bogus output for now
+	  instead of actually trying to handle it.  If they manage to
+	  browbeat the world into doing things their way, as they so
+	  often do, I'll consider at some point in the future adding
+	  support for multi-line XHDR field response.  Grr.
+	  - jik 12/16/97
 	*/
-      while (*line && isspace((unsigned char)*line))
-	line++;
-      for (ptr = strchr(line, '\0') - 1; (ptr >= line) &&
-	     isspace((unsigned char)*ptr);
-	   *ptr-- = '\0') /* empty */;
-      if (!required && (strcmp(line, "(none)") == 0))
-	continue;
+	if (isspace((unsigned char)*message))
+	  continue;
 
-      art = artStructGet(newsgroup, number, True);
-      if (offset != (unsigned)-1)
-	*(char **)((char *)art + offset) =
-	  XtNewString(line);
-      if (fixfunction)
-	*(char **)((char *) art + fixed_offset) =
-	  (*fixfunction)(newsgroup, number, line);
-      artStructSet(newsgroup, &art);
+	count++;
+
+	/*
+	 * message is of the form:
+	 *
+	 *    Number value
+	 *
+	 * must get the number since not all articles will be returned
+	 */
+
+	number = atol(message);
+	line = index(message, ' ');
+	if (! (number && line)) {
+	  mesgPane(XRN_SERIOUS, pane_name, MALFORMED_XHDR_RESPONSE_MSG,
+		   command, message);
+	  /* Let's hope that even though the server is sending us bogus
+	     data, it'll eventually sent a correct ".\r\n" line to
+	     terminate the output of the command. */
+	  continue;
+	}
+
+	/*
+	  Strip leading and trailing spaces.
+	*/
+	while (*line && isspace((unsigned char)*line))
+	  line++;
+	for (ptr = strchr(line, '\0') - 1; (ptr >= line) &&
+	       isspace((unsigned char)*ptr);
+	     *ptr-- = '\0') /* empty */;
+	if (!required && (strcmp(line, "(none)") == 0))
+	  continue;
+
+	xhdr_fields |= field_bit;
+	art = artStructGet(newsgroup, number, True);
+	if (offset != (unsigned)-1)
+	  *(char **)((char *)art + offset) =
+	    XtNewString(line);
+	if (fixfunction)
+	  *(char **)((char *) art + fixed_offset) =
+	    (*fixfunction)(newsgroup, number, line);
+	artStructSet(newsgroup, &art);
+      }
+
+      /* This is really hideous.  The problem is that with some NNTP
+	 server implementations, if you send an XHDR request for a
+	 field for which they don't support XHDR, instead of giving
+	 you an error code indicating that they don't support XHDR,
+	 instead they just giveyou a list of article numbers with
+	 (none) for each of them.  Therefore, if we've just tried to
+	 retrieve a particular header from a set of articles and we
+	 didn't find that header in *any* of them, there's a chance
+	 that the problem is actually that the NNTP server doesn't
+	 support XHDR on that header, rather than that the header
+	 doesn't exist in the articles.  So we need to try using HEAD
+	 instead of XHDR.  If we succeed, then the bit gets added to
+	 no_xhdr_fields and we'll know to do it automatically in the
+	 future. */
+      if (field_bit && ! (xhdr_fields & field_bit)) {
+	trying_again = True;
+	goto try_again;
+      }
     }
+
     for (number = first; number <= last; number++) {
       struct article copy;
       Boolean changed = False;
 
       art = artStructGet(newsgroup, number, False);
       copy = *art;
-      if ((offset != (unsigned)-1) && ! *(char **)((char *) art + offset))
+      if ((offset != (unsigned)-1) && ! *(char **)((char *) art + offset)) {
 	if (required)
 	  goto unavail;
 	else {
 	  *(char **)((char *)&copy + offset) = XtNewString("");
 	  changed = True;
 	}
-      if (fixfunction && ! *(char **)((char *)art + fixed_offset))
+      }
+      if (fixfunction && ! *(char **)((char *)art + fixed_offset)) {
 	if (required)
 	  goto unavail;
 	else {
 	  *(char **)((char *)&copy + fixed_offset) = XtNewString("");
 	  changed = True;
 	}
+      }
       if (changed)
 	artStructReplace(newsgroup, &art, &copy, number);
+      else
+	ART_STRUCT_UNLOCK;
       continue;
     unavail:
       CLEAR_ALL_NO_FREE(&copy);
@@ -1631,7 +1799,6 @@ static Boolean getlist(
     }
     first = last + 1;
 
-  next_iteration:
     if (max)
       break;
   }
@@ -1663,7 +1830,7 @@ Boolean getsubjectlist(
     offset = (char *)&foo.subject - (char *)&foo;
 
     return getlist(newsgroup, artfirst, artlast, unreadonly, max,
-		   "subject", offset, 0, 0, True);
+		   "subject", 0, offset, 0, 0, True);
 }
 
 Boolean getnewsgroupslist(
@@ -1685,7 +1852,7 @@ Boolean getnewsgroupslist(
     offset = (char *)&foo.newsgroups - (char *)&foo;
 
     return getlist(newsgroup, artfirst, artlast, unreadonly, max,
-		   "newsgroups", offset, 0, 0, True);
+		   "newsgroups", FETCH_NEWSGROUPS, offset, 0, 0, True);
 }
 
 Boolean getxreflist(
@@ -1707,7 +1874,7 @@ Boolean getxreflist(
     offset = (char *)&foo.xref - (char *)&foo;
 
     return getlist(newsgroup, artfirst, artlast, unreadonly, max,
-		   "xref", offset, 0, 0, False);
+		   "xref", FETCH_XREF, offset, 0, 0, False);
 }
 
 Boolean getapprovedlist(
@@ -1729,7 +1896,7 @@ Boolean getapprovedlist(
     offset = (char *)&foo.approved - (char *)&foo;
 
     return getlist(newsgroup, artfirst, artlast, unreadonly, max,
-		   "approved", offset, 0, 0, False);
+		   "approved", FETCH_APPROVED, offset, 0, 0, False);
 }
 
 Boolean getdatelist(
@@ -1751,7 +1918,7 @@ Boolean getdatelist(
     offset = (char *)&foo.date - (char *)&foo;
 
     return getlist(newsgroup, artfirst, artlast, unreadonly, max,
-		   "date", offset, 0, 0, True);
+		   "date", FETCH_DATES, offset, 0, 0, True);
 }
 
 Boolean getidlist(
@@ -1773,7 +1940,7 @@ Boolean getidlist(
     offset = (char *)&foo.id - (char *)&foo;
 
     return getlist(newsgroup, artfirst, artlast, unreadonly, max,
-		   "message-id", offset, 0, 0, True);
+		   "message-id", FETCH_IDS, offset, 0, 0, True);
 }
 
 Boolean getreflist(
@@ -1795,7 +1962,7 @@ Boolean getreflist(
     offset = (char *)&foo.references - (char *)&foo;
 
     return getlist(newsgroup, artfirst, artlast, unreadonly, max,
-		   "references", offset, 0, 0, False);
+		   "references", FETCH_REFS, offset, 0, 0, False);
 }
 
 static char *authorFixFunction(newsgroup, artnum, message)
@@ -1916,7 +2083,7 @@ Boolean getauthorlist(
     fixed_offset = (char *)&foo.author - (char *)&foo;
 
     return getlist(newsgroup, artfirst, artlast, unreadonly, max,
-		   "from", offset, authorFixFunction, fixed_offset, True);
+		   "from", 0, offset, authorFixFunction, fixed_offset, True);
 }
 
 
@@ -1925,32 +2092,20 @@ static char *linesFixFunction(newsgroup, artnum, numoflines)
     art_num artnum;
     char *numoflines;
 {
-    char *end;
-    int lcv;
-
-    /* Ignore extra whitespace at the beginning of the field */
-    for ( ; (*numoflines == ' ') || (*numoflines == '\t'); numoflines++)
-	/* empty */;
-
-    if ((end = index(numoflines, ' ')) != NIL(char)) {
-	*end = '\0';
-    }
+    char *buf;
 
     if (numoflines[0] != '(') {
-	numoflines[utStrlen(numoflines)+1] = '\0';
-	numoflines[utStrlen(numoflines)] = ']';
-	for (lcv = utStrlen(numoflines); lcv >= 0; lcv--) {
-	    numoflines[lcv+1] = numoflines[lcv];
-	}
-	numoflines[0] = '[';
+      buf = XtMalloc(strlen(numoflines) + 3);
+      (void) sprintf(buf, "[%s]", numoflines);
     } else {
-	numoflines[0] = '[';
-	numoflines[utStrlen(numoflines)-1] = ']';
+      buf = XtNewString(numoflines);
+      *buf = '[';
+      *(strchr(buf, '\0')-1) = ']';
     }
-    if (strcmp(numoflines, "[none]") == 0) {
-	(void) strcpy(numoflines, "[?]");
+    if (strcmp(buf, "[none]") == 0) {
+	(void) strcpy(buf, "[?]");
     }
-    return XtNewString(numoflines);
+    return buf;
 }
 
 Boolean getlineslist(
@@ -1975,7 +2130,7 @@ Boolean getlineslist(
     offset = (char *)&foo.lines - (char *)&foo;
 
     return getlist(newsgroup, artfirst, artlast, unreadonly, max,
-		   "lines", (unsigned)-1, linesFixFunction, offset, False);
+		   "lines", 0, (unsigned)-1, linesFixFunction, offset, False);
 }
 
 #ifndef INEWS
@@ -2032,7 +2187,7 @@ static char ** wrapText(ptr)
     char *ptr;
 {
      int c = 0;		/* current line length */
-     char **lines, *this_line;
+     char **lines = NULL, *this_line;
      unsigned int num_lines = 0;
      int breakAt = app_resources.breakLength;
      int maxLength;
@@ -2042,8 +2197,6 @@ static char ** wrapText(ptr)
        maxLength = app_resources.breakLength;
      }
 
-     lines = (char **) XtMalloc((Cardinal) 0);
-
      if (app_resources.breakLength && app_resources.lineLength) {
 	 /*
 	  * Do text wrapping.
@@ -2318,11 +2471,7 @@ int postArticle(article, mode, ErrMsg)
 	struct stat buf;
 	char temp[1024];
 
-#ifndef INN	
 	(void) sprintf(temp, "\n\ninews exit value: %d\n", exitstatus);
-#else
-	temp[0] = '\0';
-#endif /* INN */	
 	if ((filefp = fopen(tempfile, "r")) != NULL) {
 	    if (fstat(fileno(filefp), &buf) != -1) {
 		p = XtMalloc(buf.st_size + utStrlen(temp) + 10);
@@ -2358,12 +2507,18 @@ int postArticle(article, mode, ErrMsg)
 /*
  * get XHDR information for a single article
  */
-static char *xhdr_single _ARGUMENTS((struct newsgroup *, char *, long *));
+static char *xhdr_single _ARGUMENTS((struct newsgroup *, char *, long *, Boolean));
 
-static char *xhdr_single(newsgroup, buffer, error_code)
-    struct newsgroup *newsgroup;
-    char *buffer;
-    long *error_code;
+static char *xhdr_single(
+			 _ANSIDECL(struct newsgroup *,	newsgroup),
+			 _ANSIDECL(char *,		buffer),
+			 _ANSIDECL(long *,		error_code),
+			 _ANSIDECL(Boolean,		silent_error)
+			 )
+     _KNRDECL(struct newsgroup *,	newsgroup)
+     _KNRDECL(char *,		buffer)
+     _KNRDECL(long *,		error_code)
+     _KNRDECL(Boolean,		silent_error)
 {
     char *message, *ptr;
 
@@ -2380,8 +2535,10 @@ static char *xhdr_single(newsgroup, buffer, error_code)
     /* check for errors */
     if (*message != CHAR_OK) {
       if ((*error_code = atol(message)) != ERR_NOART) {
-	fprintf(stderr, "NNTP error: %s\n", message);
-	mesgPane(XRN_SERIOUS, 0, XHDR_ERROR_MSG);
+	if (! silent_error) {
+	  fprintf(stderr, "NNTP error: %s\n", message);
+	  mesgPane(XRN_SERIOUS, 0, XHDR_ERROR_MSG);
+	}
       }
       return NULL;
     }
@@ -2422,8 +2579,149 @@ static char *xhdr_single(newsgroup, buffer, error_code)
 
     return ptr;
 }
+
+/*
+  If this hasn't already been done:
+
+  Use the HEAD command to retrieve the header of the specified
+  article.  Create a new AVL tree to store the parsed headers.  The
+  keys in the tree are the header field names, turned to lower case,
+  and the values in the tree are the values of the matching header
+  lines, with newlines converted into spaces and the final newline
+  omitted.  ONLY THE FIRST HEADER FIELD WITH EACH FIELD NAME IS
+  STORED, e.g., if there are two "X-Foo:" headers in the message, only
+  the first one is stored and retrieved.
+*/
   
-#ifdef DONT_USE_XHDR_FOR_A_SINGLE_ITEM
+static void get_article_headers(newsgroup, article)
+     struct newsgroup *newsgroup;
+     art_num article;
+{
+  struct article *art;
+  char buffer[BUFFER_SIZE], *message, *header;
+  int header_len, header_size;
+  file_cache_file *cache_file;
+  FILE *fp;
+  avl_tree *headers;
+
+  art = artStructGet(newsgroup, article, False);
+  ART_STRUCT_UNLOCK;
+  if (art->headers)
+    return;
+
+  art = artStructGet(newsgroup, article, True);
+  art->headers = headers = avl_init_table(strcmp);
+
+  header_size = 80;
+  header = XtMalloc(header_size);
+  header_len = 0;
+
+  artStructSet(newsgroup, &art);
+
+  if (get_base_article(newsgroup, article, &cache_file, False) > 0) {
+    Boolean sawnl = False;
+
+    if (! (fp = fopen(file_cache_file_name(FileCache, *cache_file), "r"))) {
+      file_cache_file_unlock(FileCache, *cache_file);
+      return;
+    }
+
+    for (;;) {
+      int len;
+
+      if (header_size - header_len < 2) {
+	header_size *= 2;
+	header = XtRealloc(header, header_size);
+      }
+
+      if (! fgets(header + header_len, header_size - header_len, fp))
+	break;
+
+      len = strlen(header + header_len);
+      header_len += len;
+
+      if (header[header_len-1] != '\n') {
+	sawnl = False;
+	continue;
+      }
+
+      if (sawnl && (len == 1))
+	break;
+      sawnl = True;
+    }
+
+    (void) fclose(fp);
+    file_cache_file_unlock(FileCache, *cache_file);
+  }
+  else {
+    if (SETNEWSGROUP(newsgroup)) {
+      return;
+    }
+
+    (void) sprintf(buffer, "HEAD %ld", article);
+    put_server(buffer);
+    message = get_data_from_server(True);
+
+    check_server_response(buffer, &message);
+
+    if (*message != CHAR_OK) {
+      /* can't get header */
+      return;
+    }
+
+    for (;;) {
+      int len;
+      message = get_data_from_server(True);
+      if (! (*message && strcmp(message, "."))) {
+	break;
+      }
+      if (*message == '.')
+	message++;
+      len = strlen(message);
+      while (header_len + len + 2 > header_size) {
+	header_size *= 2;
+	header = XtRealloc(header, header_size);
+      }
+      (void) strcpy(header + header_len, message);
+      *(header + header_len + len) = '\n';
+      *(header + header_len + len + 1) = '\0';
+      header_len += len + 1;
+    }
+  }
+  
+  header = XtRealloc(header, header_len + 1);
+
+  avl_insert(headers, ART_HEADERS_BASE, header);
+
+  while (*header) {
+    char *ptr, *name, *value;
+
+    if (! (value = strchr(name = header, ':')))
+      break;
+
+    for (ptr = name; ptr < value; ptr++)
+      if (isupper(*ptr))
+	*ptr = tolower(*ptr);
+
+    *value++ = '\0';
+
+    for (ptr = strchr(value, '\n'); ptr; ptr = strchr(ptr, '\n')) {
+      if (ptr[1] != ' ' && ptr[1] != '\t') {
+	*ptr = '\0';
+	header = ptr + 1;
+	if (! avl_lookup(headers, name, 0)) {
+	  while (*value == ' ' || *value == '\t')
+	    value++;
+	  while (--ptr >= value && (*ptr == ' ' || *ptr == '\t'))
+	    *ptr = '\0';
+	  avl_insert(headers, name, value);
+	}
+	break;
+      }
+      *ptr = ' ';
+    }
+  }
+}
 
 /*
  * get header information about 'article'
@@ -2436,60 +2734,37 @@ void xhdr(newsgroup, article, field, string)
     char *field;
     char **string;
 {
-    char buffer[BUFFER_SIZE], *message, *ptr, *cmp, *found = 0;
+  struct article *art;
+  char *value;
 
-    if (SETNEWSGROUP(newsgroup)) {
-	*string = 0;
-	return;
-    }
+  get_article_headers(newsgroup, article);
 
-    /*
-     * In some implementations of NNTP, the XHDR request on a
-     * single article can be *very* slow, so we do a HEAD request
-     * instead and just search for the appropriate field.
-     */
-    (void) sprintf(buffer, "HEAD %ld", article);
-    put_server(buffer);
-    message = get_data_from_server(True);
-
-    check_server_response(buffer, &message);
-
-    if (*message != CHAR_OK) {
-	/* can't get header */
-	*string = NIL(char);
-	return;
-    }
-
-    for (;;) {
-        message = get_data_from_server(True);
-
-	/* the header information is ended by a '.' on a line by itself */
-	if (message[0] == '.')
-	    break;
-
-	if (!found) {
-	    for (ptr = message, cmp = field; *ptr; ptr++, cmp++) {
-		/* used to be 'mklower' */
-		if (tolower(*cmp) != tolower(*ptr))
-		    break;
-	    }
-	    if (*cmp == 0 && *ptr == ':') {
-		while (*++ptr == ' ')
-		    ;
-		found = XtNewString(ptr);
-	    }
-	}
-    }
-
-    if (found)
-	*string = found;
-    else
-	*string = NIL(char);
-
-    return;
+  art = artStructGet(newsgroup, article, False);
+  ART_STRUCT_UNLOCK;
+  if (art->headers && avl_lookup(art->headers, field, &value))
+    *string = XtNewString(value);
+  else
+    *string = 0;
 }
 
-#else
+/* We're not currently using XHDR to retrieve information about single
+   articles, for two reasons: (1) whenever we're doing that, we're
+   probably going to need multiple fields for the same article; (2) in
+   fact we've probably already retrieved the article, and parsing its
+   header fields is cheaper than a round trip to the server; (3) many
+   so-called "modern" NNTP servers don't support XHDR for all header
+   fields; they only return valid XHDR data for fields that are in the
+   overview database.  This sucks, but we have to cope with it
+   anyway.
+
+   However, we're preserving this functionality here, in case we end
+   up needing to use it later, rather than removing it completely.  If
+   we do end up using this functionality at any point in the future,
+   the function needs to be updated so that it knows how to fall back
+   on checking the header if the XHDR command fails.
+*/
+
+#if 0
 
 /*
  * get header information about 'article'
@@ -2506,13 +2781,15 @@ void xhdr(newsgroup, article, field, string)
     long error_code;
 
     (void) sprintf(buffer, "XHDR %s %ld", field, article);
-    *string = xhdr_single(newsgroup, buffer, &error_code);
+    *string = xhdr_single(newsgroup, buffer, &error_code, False);
     return;
 }
+
 #endif
 
 /* Get XHDR information using an article's message ID.  The returned
-   data is allocated and should be freed by the caller. */
+   data is allocated and should be freed by the caller.  Fails
+   silently if the NNTP server doesn't support XHDR by message ID. */
   
 char *xhdr_id(newsgroup, id, field, error_code)
      struct newsgroup *newsgroup;
@@ -2523,7 +2800,7 @@ char *xhdr_id(newsgroup, id, field, error_code)
   char buffer[BUFFER_SIZE];
 
   (void) sprintf(buffer, "XHDR %s %s", field, id);
-  return xhdr_single(newsgroup, buffer, error_code);
+  return xhdr_single(newsgroup, buffer, error_code, True);
 }
 
 #ifndef POPEN_USES_INEXPENSIVE_FORK
@@ -2531,8 +2808,8 @@ char *xhdr_id(newsgroup, id, field, error_code)
 static int popen_pid = 0;
 
 FILE *xrn_popen(command, type)
-    CONST char *command;
-    CONST char *type;
+    const char *command;
+    const char *type;
 {
     int pipes[2];
     int itype = (strcmp(type, "w") == 0 ? 1 : 0);
