@@ -37,12 +37,13 @@
 #include "sort.h"
 #include "artstruct.h"
 #include "internals.h"
-#include "getdate.h"
 #include "mesg.h"
 #include "mesg_strings.h"
 #include "hash.h"
 #include "resources.h"
 
+extern time_t get_date _ARGUMENTS((char *));
+
 #define SUB_SORT_WIDTH 24
 
 
@@ -60,7 +61,7 @@ struct sort_data {
 };
 
 
-static int (*compare_function) _ARGUMENTS((qsort_arg_type, qsort_arg_type));
+static int (*compare_function) _ARGUMENTS((const void *, const void *));
 static void (*sort_list[3]) _ARGUMENTS((void *));
 
 void art_sort_parse_sortlist(orig_sort_spec)
@@ -144,6 +145,7 @@ void *art_sort_init(
     struct article *art = artStructGet(newsgroup, i, False);
     if (IS_LISTED(art))
       count++;
+    ART_STRUCT_UNLOCK;
   }
 
   data->articles = (struct sort_article *)
@@ -157,6 +159,7 @@ void *art_sort_init(
       data->articles[count].num = i;
       data->articles[count++].art = art;
     }
+    ART_STRUCT_UNLOCK;
   }
 
   return (void *) data;
@@ -224,10 +227,10 @@ char *art_sort_doit(
 }
    
 
-static int key_compare _ARGUMENTS((qsort_arg_type, qsort_arg_type));
+static int key_compare _ARGUMENTS((const void *, const void *));
 
 static int key_compare(a_p, b_p)
-     qsort_arg_type a_p, b_p;
+     const void *a_p, *b_p;
 {
   struct sort_article *a = (struct sort_article *) a_p;
   struct sort_article *b = (struct sort_article *) b_p;
@@ -350,7 +353,7 @@ void generate_date_keys(data)
 }
 
 int date_compare(a, b)
-     qsort_arg_type a, b;
+     const void *a, *b;
 {
   /*
     Casting to char * because pointer arithmetic on void * causes a
@@ -391,8 +394,8 @@ static void do_art_thread(data, table, this_art, artlist, artpos)
   int i;
   art_num *ptr;
 
-  i = (int)hash_table_retrieve(table, (void *)this_art, 0);
-  assert(i != (int)HASH_NO_VALUE);
+  i = (POINTER_NUM_TYPE)hash_table_retrieve(table, (void *)this_art, 0);
+  assert(i != (POINTER_NUM_TYPE)HASH_NO_VALUE);
 
   /* Circular article references are bogus, but unfortunately possible */
   if (! data->articles[i].sort_key)
@@ -414,7 +417,7 @@ void art_sort_by_thread(data_p)
 {
   struct sort_data *data = (struct sort_data *)data_p;
   struct article *art;
-  int i, ret;
+  POINTER_NUM_TYPE i, ret;
   hash_table_object table, done_table;
   struct sort_article *tmp_articles;
   int tmp_pos = 0;
@@ -452,7 +455,8 @@ void art_sort_by_thread(data_p)
       if (! hash_table_insert(done_table, (void *) art->parent, (void *)1, 1))
 	break;
       this_art = art->parent;
-      art = artStructGet(data->newsgroup, art->parent, False);
+      art = artStructGet(data->newsgroup, this_art, False);
+      ART_STRUCT_UNLOCK;
       assert(art);
     }
     do_art_thread(data, table, this_art, tmp_articles, &tmp_pos);
