@@ -1,7 +1,7 @@
 #include "utils.h"
 #include "config.h"
 
-#ifdef NEED_STRTOK
+#ifndef HAVE_STRTOK
 
 /*
  * Get next token from string s (NULL on 2nd, 3rd, etc. calls),
@@ -14,11 +14,11 @@ static char *scanpoint = NULL;
 				/* NULL if no token left */
 char * strtok(s, delim)
     char *s;
-    register char CONST *delim;
+    register char const *delim;
 {
 	register char *scan;
 	char *tok;
-	register CONST char *dscan;
+	register const char *dscan;
 
 	if (s == NULL && scanpoint == NULL)
 		return NULL;
